@@ -12,16 +12,18 @@ __path__ = __import__("pkgutil").extend_path(__path__, __name__)
 import logging
 
 logger = logging.getLogger("pydra")
-from ._version import get_versions
 
-__version__ = get_versions()["version"]
-del get_versions
+try:
+    from ._version import __version__
+except ImportError:
+    pass
 
 from .engine import Submitter, Workflow, AuditFlag, ShellCommandTask, DockerTask, specs
 from . import mark
 
 
 def check_latest_version():
+
     import etelemetry
 
     return etelemetry.check_available_version("nipype/pydra", __version__, lgr=logger)
