@@ -4,7 +4,8 @@ from pathlib import Path
 import json
 import attr
 from ..utils.messenger import send_message, make_message, gen_uuid, now, AuditFlag
-from .helpers import ensure_list, gather_runtime_info
+from .helpers import ensure_list, gather_runtime_info, hash_file
+from .specs import attr_fields, File, Directory
 
 
 class Audit:
@@ -18,7 +19,8 @@ class Audit:
         ----------
         audit_flags : :class:`AuditFlag`
             Base configuration of auditing.
-        messengers : :obj:`pydra.util.messenger.Messenger` or list of :class:`pydra.util.messenger.Messenger`, optional
+        messengers : :obj:`pydra.util.messenger.Messenger`
+            or list of :class:`pydra.util.messenger.Messenger`, optional
             Specify types of messenger used by Audit to send a message.
             Could be `PrintMessenger`, `FileMessenger`, or `RemoteRESTMessenger`.
         messenger_args : :obj:`dict`, optional
@@ -48,7 +50,15 @@ class Audit:
         self.odir = odir
         if self.audit_check(AuditFlag.PROV):
             self.aid = f"uid:{gen_uuid()}"
-            start_message = {"@id": self.aid, "@type": "task", "startedAtTime": now()}
+
+            user_id = f"uid:{gen_uuid()}"
+            start_message = {
+                "@id": self.aid,
+                "@type": "task",
+                "startedAtTime": now(),
+                "executedBy": user_id,
+            }
+
         os.chdir(self.odir)
         if self.audit_check(AuditFlag.PROV):
             self.audit_message(start_message, AuditFlag.PROV)
@@ -85,7 +95,7 @@ class Audit:
                 )
                 # audit resources/runtime information
                 self.eid = f"uid:{gen_uuid()}"
-                entity = attr.asdict(result.runtime)
+                entity = attr.asdict(result.runtime, recurse=False)
                 entity.update(
                     **{
                         "@id": self.eid,
@@ -159,3 +169,55 @@ class Audit:
             Boolean AND for self.oudit_flags and flag
         """
         return self.audit_flags & flag
+
+    def audit_task(self, task):
+        import subprocess as sp
+
+        label = task.name
+
+        command = task.cmdline if hasattr(task.inputs, "executable") else None
+        attr_list = attr_fields(task.inputs)
+        for attrs in attr_list:
+            if attrs.type in [File, Directory]:
+                input_name = attrs.name
+                input_path = os.path.abspath(getattr(task.inputs, input_name))
+                file_hash = hash_file(input_path)
+                entity_id = f"uid:{gen_uuid()}"
+                entity_message = {
+                    "@id": entity_id,
+                    "Label": input_name,
+                    "AtLocation": input_path,
+                    "GeneratedBy": None,
+                    "@type": "input",
+                    "digest": file_hash,
+                }
+                self.audit_message(entity_message, AuditFlag.PROV)
+
+        if command is not None:
+            cmd_name = command.split()[0]
+            software = f"{cmd_name} --version"
+            # take the first word of command as the
+            # name of the executable
+            # (this may not always be the case)
+            version_cmd = sp.run(software, shell=True, stdout=sp.PIPE).stdout.decode(
+                "utf-8"
+            )
+            try:
+                version_cmd = version_cmd.splitlines()[0]
+
+            except IndexError:
+                version_cmd = f"{cmd_name} -- Version unknown"
+
+        else:
+            version_cmd = None
+
+        start_message = {
+            "@id": self.aid,
+            "@type": "task",
+            "Label": label,
+            "Command": command,
+            "StartedAtTime": now(),
+            "AssociatedWith": version_cmd,
+        }
+
+        self.audit_message(start_message, AuditFlag.PROV)
