@@ -4,14 +4,17 @@ import attr
 import json
 import logging
 import os
+import sys
 from pathlib import Path
 import typing as ty
-from copy import deepcopy, copy
+from copy import deepcopy
+from uuid import uuid4
 
 import cloudpickle as cp
 from filelock import SoftFileLock
 import shutil
 from tempfile import mkdtemp
+from traceback import format_exception
 
 from . import state
 from . import helpers_state as hlpst
@@ -34,6 +37,7 @@ from .helpers import (
     ensure_list,
     record_error,
     hash_function,
+    PydraFileLock,
 )
 from .helpers_file import copyfile_input, template_update
 from .graph import DiGraph
@@ -80,6 +84,7 @@ class TaskBase:
         cache_dir=None,
         cache_locations=None,
         inputs: ty.Optional[ty.Union[ty.Text, File, ty.Dict]] = None,
+        cont_dim=None,
         messenger_args=None,
         messengers=None,
         rerun=False,
@@ -113,11 +118,13 @@ class TaskBase:
             TODO
         inputs : :obj:`typing.Text`, or :class:`File`, or :obj:`dict`, or `None`.
             Set particular inputs to this node.
+        cont_dim : :obj:`dict`, or `None`
+            Container dimensions for input fields,
+            if any of the container should be treated as a container
         messenger_args :
             TODO
         messengers :
             TODO
-
         """
         from .. import check_latest_version
 
@@ -163,10 +170,13 @@ class TaskBase:
 
         # checking if metadata is set properly
         self.inputs.check_metadata()
-        self.state_inputs = inputs
         # dictionary to save the connections with lazy fields
         self.inp_lf = {}
         self.state = None
+        # container dimensions provided by the user
+        self.cont_dim = cont_dim
+        # container dimension for inner input if needed (e.g. for inner splitter)
+        self._inner_cont_dim = {}
         self._output = {}
         self._result = {}
         # flag that says if node finished all jobs
@@ -184,6 +194,7 @@ class TaskBase:
         self.cache_locations = cache_locations
         self.allow_cache_override = True
         self._checksum = None
+        self._uid = uuid4().hex
         # if True the results are not checked (does not propagate to nodes)
         self.task_rerun = rerun
 
@@ -199,7 +210,7 @@ class TaskBase:
         state["input_spec"] = cp.dumps(state["input_spec"])
         state["output_spec"] = cp.dumps(state["output_spec"])
         inputs = {}
-        for k, v in attr.asdict(state["inputs"]).items():
+        for k, v in attr.asdict(state["inputs"], recurse=False).items():
             if k.startswith("_"):
                 k = k[1:]
             inputs[k] = v
@@ -235,10 +246,10 @@ class TaskBase:
 
     @property
     def checksum(self):
-        """ Calculates the unique checksum of the task.
-            Used to create specific directory name for task that are run;
-            and to create nodes checksums needed for graph checkums
-            (before the tasks have inputs etc.)
+        """Calculates the unique checksum of the task.
+        Used to create specific directory name for task that are run;
+        and to create nodes checksums needed for graph checksums
+        (before the tasks have inputs etc.)
         """
         input_hash = self.inputs.hash
         if self.state is None:
@@ -262,20 +273,30 @@ class TaskBase:
             TODO
 
         """
-        self.state.prepare_states(self.inputs)
-        self.state.prepare_inputs()
+        if is_workflow(self) and self.inputs._graph_checksums is attr.NOTHING:
+            self.inputs._graph_checksums = [nd.checksum for nd in self.graph_sorted]
+
         if state_index is not None:
             inputs_copy = deepcopy(self.inputs)
             for key, ind in self.state.inputs_ind[state_index].items():
-                setattr(
-                    inputs_copy,
-                    key.split(".")[1],
-                    getattr(inputs_copy, key.split(".")[1])[ind],
+                val = self._extract_input_el(
+                    inputs=inputs_copy, inp_nm=key.split(".")[1], ind=ind
                 )
+                setattr(inputs_copy, key.split(".")[1], val)
+            # setting files_hash again in case it was cleaned by setting specific element
+            # that might be important for outer splitter of input variable with big files
+            # the file can be changed with every single index even if there are only two files
+            inputs_copy.files_hash = self.inputs.files_hash
             input_hash = inputs_copy.hash
+            # updating self.inputs.files_hash, so big files hashes
+            # doesn't have to be recompute for the next element
+            for key, val in inputs_copy.files_hash.items():
+                if val:
+                    self.inputs.files_hash[key].update(val)
             if is_workflow(self):
                 con_hash = hash_function(self._connections)
-                hash_list = [input_hash, con_hash]
+                # TODO: hash list is not used
+                hash_list = [input_hash, con_hash]  # noqa: F841
                 checksum_ind = create_checksum(
                     self.__class__.__name__, self._checksum_wf(input_hash)
                 )
@@ -284,10 +305,21 @@ class TaskBase:
             return checksum_ind
         else:
             checksum_list = []
+            if not hasattr(self.state, "inputs_ind"):
+                self.state.prepare_states(self.inputs, cont_dim=self.cont_dim)
+                self.state.prepare_inputs()
             for ind in range(len(self.state.inputs_ind)):
                 checksum_list.append(self.checksum_states(state_index=ind))
             return checksum_list
 
+    @property
+    def uid(self):
+        """the unique id number for the task
+        It will be used to create unique names for slurm scripts etc.
+        without a need to run checksum
+        """
+        return self._uid
+
     def set_state(self, splitter, combiner=None):
         """
         Set a particular state on this task.
@@ -310,9 +342,33 @@ class TaskBase:
 
     @property
     def output_names(self):
-        """Get the names of the outputs generated by the task."""
+        """Get the names of the outputs from the task's output_spec
+        (not everything has to be generated, see generated_output_names).
+        """
         return [f.name for f in attr.fields(make_klass(self.output_spec))]
 
+    @property
+    def generated_output_names(self):
+        """Get the names of the outputs generated by the task.
+        If the spec doesn't have generated_output_names method,
+        it uses output_names.
+        The results depends on the input provided to the task
+        """
+        output_klass = make_klass(self.output_spec)
+        if hasattr(output_klass, "generated_output_names"):
+            output = output_klass(**{f.name: None for f in attr.fields(output_klass)})
+            # using updated input (after filing the templates)
+            _inputs = deepcopy(self.inputs)
+            modified_inputs = template_update(_inputs, self.output_dir)
+            if modified_inputs:
+                _inputs = attr.evolve(_inputs, **modified_inputs)
+
+            return output.generated_output_names(
+                inputs=_inputs, output_dir=self.output_dir
+            )
+        else:
+            return self.output_names
+
     @property
     def can_resume(self):
         """Whether the task accepts checkpoint-restart."""
@@ -355,74 +411,121 @@ class TaskBase:
             return [self._cache_dir / checksum for checksum in self.checksum_states()]
         return self._cache_dir / self.checksum
 
-    def __call__(self, submitter=None, plugin=None, rerun=False, **kwargs):
+    @property
+    def cont_dim(self):
+        # adding inner_cont_dim to the general container_dimension provided by the users
+        cont_dim_all = deepcopy(self._cont_dim)
+        for k, v in self._inner_cont_dim.items():
+            cont_dim_all[k] = cont_dim_all.get(k, 1) + v
+        return cont_dim_all
+
+    @cont_dim.setter
+    def cont_dim(self, cont_dim):
+        if cont_dim is None:
+            self._cont_dim = {}
+        else:
+            self._cont_dim = cont_dim
+
+    def __call__(
+        self, submitter=None, plugin=None, plugin_kwargs=None, rerun=False, **kwargs
+    ):
         """Make tasks callable themselves."""
         from .submitter import Submitter
 
         if submitter and plugin:
             raise Exception("Specify submitter OR plugin, not both")
-        plugin = plugin or self.plugin
-        if plugin:
-            submitter = Submitter(plugin=plugin)
-        elif self.state:
-            submitter = Submitter()
+        elif submitter:
+            pass
+        # if there is plugin provided or the task is a Workflow or has a state,
+        # the submitter will be created using provided plugin, self.plugin or "cf"
+        elif plugin or self.state or is_workflow(self):
+            plugin = plugin or self.plugin or "cf"
+            if plugin_kwargs is None:
+                plugin_kwargs = {}
+            submitter = Submitter(plugin=plugin, **plugin_kwargs)
 
         if submitter:
             with submitter as sub:
+                self.inputs = attr.evolve(self.inputs, **kwargs)
                 res = sub(self)
-        else:
-            if is_workflow(self):
-                raise NotImplementedError(
-                    "TODO: linear workflow execution - assign submitter or plugin for now"
-                )
+        else:  # tasks without state could be run without a submitter
             res = self._run(rerun=rerun, **kwargs)
         return res
 
+    def _modify_inputs(self):
+        """Update and preserve a Task's original inputs"""
+        orig_inputs = {
+            k: deepcopy(v) for k, v in attr.asdict(self.inputs, recurse=False).items()
+        }
+        map_copyfiles = copyfile_input(self.inputs, self.output_dir)
+        modified_inputs = template_update(
+            self.inputs, self.output_dir, map_copyfiles=map_copyfiles
+        )
+        if modified_inputs:
+            self.inputs = attr.evolve(self.inputs, **modified_inputs)
+        return orig_inputs
+
+    def _populate_filesystem(self, checksum, output_dir):
+        """
+        Invoked immediately after the lockfile is generated, this function:
+        - Creates the cache file
+        - Clears existing outputs if `can_resume` is False
+        - Generates a fresh output directory
+
+        Created as an attempt to simplify overlapping `Task`|`Workflow` behaviors.
+        """
+        # adding info file with the checksum in case the task was cancelled
+        # and the lockfile has to be removed
+        with open(self.cache_dir / f"{self.uid}_info.json", "w") as jsonfile:
+            json.dump({"checksum": checksum}, jsonfile)
+        if not self.can_resume and output_dir.exists():
+            shutil.rmtree(output_dir)
+        output_dir.mkdir(parents=False, exist_ok=self.can_resume)
+
     def _run(self, rerun=False, **kwargs):
         self.inputs = attr.evolve(self.inputs, **kwargs)
         self.inputs.check_fields_input_spec()
+
         checksum = self.checksum
+        output_dir = self.output_dir
         lockfile = self.cache_dir / (checksum + ".lock")
         # Eagerly retrieve cached - see scenarios in __init__()
         self.hooks.pre_run(self)
-        # TODO add signal handler for processes killed after lock acquisition
+        logger.debug(f"'%s' is attempting to acquire lock on %s", self.name, lockfile)
         with SoftFileLock(lockfile):
             if not (rerun or self.task_rerun):
                 result = self.result()
-                if result is not None:
+                if result is not None and not result.errored:
                     return result
-            # Let only one equivalent process run
-            odir = self.output_dir
-            if not self.can_resume and odir.exists():
-                shutil.rmtree(odir)
             cwd = os.getcwd()
-            odir.mkdir(parents=False, exist_ok=True if self.can_resume else False)
-            orig_inputs = attr.asdict(self.inputs)
-            map_copyfiles = copyfile_input(self.inputs, self.output_dir)
-            modified_inputs = template_update(
-                self.inputs, self.output_dir, map_copyfiles=map_copyfiles
-            )
-            if modified_inputs:
-                self.inputs = attr.evolve(self.inputs, **modified_inputs)
-            self.audit.start_audit(odir)
+            self._populate_filesystem(checksum, output_dir)
+            orig_inputs = self._modify_inputs()
             result = Result(output=None, runtime=None, errored=False)
             self.hooks.pre_run_task(self)
+            self.audit.start_audit(odir=output_dir)
+            if self.audit.audit_check(AuditFlag.PROV):
+                self.audit.audit_task(task=self)
             try:
                 self.audit.monitor()
                 self._run_task()
-                result.output = self._collect_outputs(output_dir=odir)
-            except Exception as e:
-                record_error(self.output_dir, e)
+                result.output = self._collect_outputs(output_dir=output_dir)
+            except Exception:
+                etype, eval, etr = sys.exc_info()
+                traceback = format_exception(etype, eval, etr)
+                record_error(output_dir, error=traceback)
                 result.errored = True
                 raise
             finally:
                 self.hooks.post_run_task(self, result)
                 self.audit.finalize_audit(result)
-                save(odir, result=result, task=self)
+                save(output_dir, result=result, task=self)
+                self.output_ = None
+                # removing the additional file with the chcksum
+                (self.cache_dir / f"{self.uid}_info.json").unlink()
                 # # function etc. shouldn't change anyway, so removing
-                orig_inputs = dict(
-                    (k, v) for (k, v) in orig_inputs.items() if not k.startswith("_")
-                )
+                orig_inputs = {
+                    k: v for k, v in orig_inputs.items() if not k.startswith("_")
+                }
                 self.inputs = attr.evolve(self.inputs, **orig_inputs)
                 os.chdir(cwd)
         self.hooks.post_run(self, result)
@@ -432,12 +535,14 @@ class TaskBase:
         run_output = self.output_
         output_klass = make_klass(self.output_spec)
         output = output_klass(**{f.name: None for f in attr.fields(output_klass)})
-        other_output = output.collect_additional_outputs(self.inputs, output_dir)
+        other_output = output.collect_additional_outputs(
+            self.inputs, output_dir, run_output
+        )
         return attr.evolve(output, **run_output, **other_output)
 
-    def split(self, splitter, overwrite=False, **kwargs):
+    def split(self, splitter, overwrite=False, cont_dim=None, **kwargs):
         """
-        Run this task parametrically over lists of splitted inputs.
+        Run this task parametrically over lists of split inputs.
 
         Parameters
         ----------
@@ -445,6 +550,10 @@ class TaskBase:
             TODO
         overwrite : :obj:`bool`
             TODO
+        cont_dim : :obj:`dict`
+            Container dimensions for specific inputs, used in the splitter.
+            If input name is not in cont_dim, it is assumed that the input values has
+            a container dimension of 1, so only the most outer dim will be used for splitting.
 
         """
         splitter = hlpst.add_name_splitter(splitter, self.name)
@@ -454,9 +563,11 @@ class TaskBase:
                 "splitter has been already set, "
                 "if you want to overwrite it - use overwrite=True"
             )
+        if cont_dim:
+            for key, vel in cont_dim.items():
+                self._cont_dim[f"{self.name}.{key}"] = vel
         if kwargs:
             self.inputs = attr.evolve(self.inputs, **kwargs)
-            self.state_inputs = kwargs
         if not self.state or splitter != self.state.splitter:
             self.set_state(splitter)
         return self
@@ -498,18 +609,38 @@ class TaskBase:
             self.set_state(splitter=self.state.splitter, combiner=self.combiner)
             return self
 
+    def _extract_input_el(self, inputs, inp_nm, ind):
+        """
+        Extracting element of the inputs taking into account
+        container dimension of the specific element that can be set in self.cont_dim.
+        If input name is not in cont_dim, it is assumed that the input values has
+        a container dimension of 1, so only the most outer dim will be used for splitting.
+        If
+        """
+        if f"{self.name}.{inp_nm}" in self.cont_dim:
+            return list(
+                hlpst.flatten(
+                    ensure_list(getattr(inputs, inp_nm)),
+                    max_depth=self.cont_dim[f"{self.name}.{inp_nm}"],
+                )
+            )[ind]
+        else:
+            return getattr(inputs, inp_nm)[ind]
+
     def get_input_el(self, ind):
         """Collect all inputs required to run the node (for specific state element)."""
         if ind is not None:
-            # TODO: doesnt work properly for more cmplicated wf (check if still an issue)
+            # TODO: doesn't work properly for more cmplicated wf (check if still an issue)
             state_dict = self.state.states_val[ind]
             input_ind = self.state.inputs_ind[ind]
             inputs_dict = {}
             for inp in set(self.input_names):
                 if f"{self.name}.{inp}" in input_ind:
-                    inputs_dict[inp] = getattr(self.inputs, inp)[
-                        input_ind[f"{self.name}.{inp}"]
-                    ]
+                    inputs_dict[inp] = self._extract_input_el(
+                        inputs=self.inputs,
+                        inp_nm=inp,
+                        ind=input_ind[f"{self.name}.{inp}"],
+                    )
                 else:
                     inputs_dict[inp] = getattr(self.inputs, inp)
             return state_dict, inputs_dict
@@ -520,11 +651,11 @@ class TaskBase:
             return None, inputs_dict
 
     def pickle_task(self):
-        """ Pickling the tasks with full inputs"""
+        """Pickling the tasks with full inputs"""
         pkl_files = self.cache_dir / "pkl_files"
         pkl_files.mkdir(exist_ok=True, parents=True)
-        task_main_path = pkl_files / f"{self.name}_{self.checksum}_task.pklz"
-        save(task_path=pkl_files, task=self, name_prefix=f"{self.name}_{self.checksum}")
+        task_main_path = pkl_files / f"{self.name}_{self.uid}_task.pklz"
+        save(task_path=pkl_files, task=self, name_prefix=f"{self.name}_{self.uid}")
         return task_main_path
 
     @property
@@ -611,7 +742,8 @@ class TaskBase:
                     return self._combined_output(return_inputs=return_inputs)
                 else:
                     results = []
-                    for checksum in self.checksum_states():
+                    for ind in range(len(self.state.inputs_ind)):
+                        checksum = self.checksum_states(state_index=ind)
                         result = load_result(checksum, self.cache_locations)
                         if result is None:
                             return None
@@ -665,6 +797,64 @@ class TaskBase:
                 task._reset()
 
 
+def _sanitize_input_spec(
+    input_spec: ty.Union[SpecInfo, ty.List[str]],
+    wf_name: str,
+) -> SpecInfo:
+    """Makes sure the provided input specifications are valid.
+
+    If the input specification is a list of strings, this will
+    build a proper SpecInfo object out of it.
+
+    Parameters
+    ----------
+    input_spec : SpecInfo or List[str]
+        Input specification to be sanitized.
+
+    wf_name : str
+        The name of the workflow for which the input specifications
+        are sanitized.
+
+    Returns
+    -------
+    input_spec : SpecInfo
+        Sanitized input specifications.
+
+    Raises
+    ------
+    ValueError
+        If provided `input_spec` is None.
+    """
+    graph_checksum_input = ("_graph_checksums", ty.Any)
+    if input_spec:
+        if isinstance(input_spec, SpecInfo):
+            if not any([x == BaseSpec for x in input_spec.bases]):
+                raise ValueError("Provided SpecInfo must have BaseSpec as it's base.")
+            if "_graph_checksums" not in {f[0] for f in input_spec.fields}:
+                input_spec.fields.insert(0, graph_checksum_input)
+            return input_spec
+        else:
+            return SpecInfo(
+                name="Inputs",
+                fields=[graph_checksum_input]
+                + [
+                    (
+                        nm,
+                        attr.ib(
+                            type=ty.Any,
+                            metadata={
+                                "help_string": f"{nm} input from {wf_name} workflow"
+                            },
+                        ),
+                    )
+                    for nm in input_spec
+                ],
+                bases=(BaseSpec,),
+            )
+    else:
+        raise ValueError(f"Empty input_spec provided to Workflow {wf_name}.")
+
+
 class Workflow(TaskBase):
     """A composite task with structure of computational graph."""
 
@@ -674,7 +864,8 @@ class Workflow(TaskBase):
         audit_flags: AuditFlag = AuditFlag.NONE,
         cache_dir=None,
         cache_locations=None,
-        input_spec: ty.Optional[ty.Union[ty.List[ty.Text], SpecInfo, BaseSpec]] = None,
+        input_spec: ty.Optional[ty.Union[ty.List[ty.Text], SpecInfo]] = None,
+        cont_dim=None,
         messenger_args=None,
         messengers=None,
         output_spec: ty.Optional[ty.Union[SpecInfo, BaseSpec]] = None,
@@ -698,6 +889,9 @@ class Workflow(TaskBase):
             TODO
         inputs : :obj:`typing.Text`, or :class:`File`, or :obj:`dict`, or `None`.
             Set particular inputs to this node.
+        cont_dim : :obj:`dict`, or `None`
+            Container dimensions for input fields,
+            if any of the container should be treated as a container
         messenger_args :
             TODO
         messengers :
@@ -706,27 +900,8 @@ class Workflow(TaskBase):
             TODO
 
         """
-        if input_spec:
-            if isinstance(input_spec, BaseSpec):
-                self.input_spec = input_spec
-            else:
-                self.input_spec = SpecInfo(
-                    name="Inputs",
-                    fields=[("_graph_checksums", ty.Any)]
-                    + [
-                        (
-                            nm,
-                            attr.ib(
-                                type=ty.Any,
-                                metadata={
-                                    "help_string": f"{nm} input from {name} workflow"
-                                },
-                            ),
-                        )
-                        for nm in input_spec
-                    ],
-                    bases=(BaseSpec,),
-                )
+        self.input_spec = _sanitize_input_spec(input_spec, name)
+
         self.output_spec = output_spec
 
         if name in dir(self):
@@ -738,6 +913,7 @@ class Workflow(TaskBase):
         super().__init__(
             name=name,
             inputs=kwargs,
+            cont_dim=cont_dim,
             cache_dir=cache_dir,
             cache_locations=cache_locations,
             audit_flags=audit_flags,
@@ -775,10 +951,10 @@ class Workflow(TaskBase):
 
     @property
     def checksum(self):
-        """ Calculates the unique checksum of the task.
-            Used to create specific directory name for task that are run;
-            and to create nodes checksums needed for graph checkums
-            (before the tasks have inputs etc.)
+        """Calculates the unique checksum of the task.
+        Used to create specific directory name for task that are run;
+        and to create nodes checksums needed for graph checksums
+        (before the tasks have inputs etc.)
         """
         # if checksum is called before run the _graph_checksums is not ready
         if is_workflow(self) and self.inputs._graph_checksums is attr.NOTHING:
@@ -797,8 +973,8 @@ class Workflow(TaskBase):
         return self._checksum
 
     def _checksum_wf(self, input_hash, with_splitter=False):
-        """ creating hash value for workflows
-            includes connections and splitter if with_splitter is True
+        """creating hash value for workflows
+        includes connections and splitter if with_splitter is True
         """
         connection_hash = hash_function(self._connections)
         hash_list = [input_hash, connection_hash]
@@ -849,6 +1025,7 @@ class Workflow(TaskBase):
             If True, `add_edges_description` is run for self.graph to add
             a detailed descriptions of the connections (input/output fields names)
         """
+        # TODO: create connection is run twice
         other_states = {}
         for field in attr_fields(task.inputs):
             val = getattr(task.inputs, field.name)
@@ -865,16 +1042,27 @@ class Workflow(TaskBase):
                             (task.name, field.name, val.name, val.field)
                         )
                     logger.debug("Connecting %s to %s", val.name, task.name)
-
+                    # adding a state from the previous task to other_states
                     if (
                         getattr(self, val.name).state
                         and getattr(self, val.name).state.splitter_rpn_final
                     ):
-                        # adding a state from the previous task to other_states
-                        other_states[val.name] = (
-                            getattr(self, val.name).state,
-                            field.name,
-                        )
+                        # variables that are part of inner splitters should be treated as a containers
+                        if (
+                            task.state
+                            and f"{task.name}.{field.name}" in task.state.splitter
+                        ):
+                            task._inner_cont_dim[f"{task.name}.{field.name}"] = 1
+                        # adding task_name: (task.state, [a field from the connection]
+                        if val.name not in other_states:
+                            other_states[val.name] = (
+                                getattr(self, val.name).state,
+                                [field.name],
+                            )
+                        else:
+                            # if the task already exist in other_state,
+                            # additional field name should be added to the list of fields
+                            other_states[val.name][1].append(field.name)
                 else:  # LazyField with the wf input
                     # connections with wf input should be added to the detailed graph description
                     if detailed:
@@ -902,64 +1090,66 @@ class Workflow(TaskBase):
                 )
 
     async def _run(self, submitter=None, rerun=False, **kwargs):
-        # self.inputs = dc.replace(self.inputs, **kwargs) don't need it?
         # output_spec needs to be set using set_output or at workflow initialization
         if self.output_spec is None:
             raise ValueError(
                 "Workflow output cannot be None, use set_output to define output(s)"
             )
-        checksum = self.checksum
-        lockfile = self.cache_dir / (checksum + ".lock")
-        # Eagerly retrieve cached
-        if not (rerun or self.task_rerun):
-            result = self.result()
-            if result is not None:
-                return result
         # creating connections that were defined after adding tasks to the wf
-        for task in self.graph.nodes:
-            # if workflow has task_rerun=True and propagate_rerun=True,
-            # it should be passed to the tasks
-            if self.task_rerun and self.propagate_rerun:
-                task.task_rerun = self.task_rerun
-                # if the task is a wf, than the propagate_rerun should be also set
-                if is_workflow(task):
-                    task.propagate_rerun = self.propagate_rerun
-            task.cache_locations = task._cache_locations + self.cache_locations
-            self.create_connections(task)
-        # TODO add signal handler for processes killed after lock acquisition
+        self._connect_and_propagate_to_tasks(
+            propagate_rerun=self.task_rerun and self.propagate_rerun
+        )
+
+        checksum = self.checksum
+        output_dir = self.output_dir
+        lockfile = self.cache_dir / (checksum + ".lock")
         self.hooks.pre_run(self)
-        with SoftFileLock(lockfile):
-            # # Let only one equivalent process run
-            odir = self.output_dir
-            if not self.can_resume and odir.exists():
-                shutil.rmtree(odir)
+        logger.debug(
+            f"'%s' is attempting to acquire lock on %s with Pydra lock",
+            self.name,
+            lockfile,
+        )
+        async with PydraFileLock(lockfile):
+            if not (rerun or self.task_rerun):
+                result = self.result()
+                if result is not None and not result.errored:
+                    return result
             cwd = os.getcwd()
-            odir.mkdir(parents=False, exist_ok=True if self.can_resume else False)
-            self.audit.start_audit(odir=odir)
+            self._populate_filesystem(checksum, output_dir)
             result = Result(output=None, runtime=None, errored=False)
             self.hooks.pre_run_task(self)
+            self.audit.start_audit(odir=output_dir)
             try:
                 self.audit.monitor()
                 await self._run_task(submitter, rerun=rerun)
                 result.output = self._collect_outputs()
-            except Exception as e:
-                record_error(self.output_dir, e)
+            except Exception:
+                etype, eval, etr = sys.exc_info()
+                traceback = format_exception(etype, eval, etr)
+                record_error(output_dir, error=traceback)
                 result.errored = True
                 self._errored = True
                 raise
             finally:
                 self.hooks.post_run_task(self, result)
                 self.audit.finalize_audit(result=result)
-                save(odir, result=result, task=self)
+                save(output_dir, result=result, task=self)
+                # removing the additional file with the chcksum
+                (self.cache_dir / f"{self.uid}_info.json").unlink()
                 os.chdir(cwd)
         self.hooks.post_run(self, result)
+        if result is None:
+            raise Exception("This should never happen, please open new issue")
         return result
 
     async def _run_task(self, submitter, rerun=False):
         if not submitter:
             raise Exception("Submitter should already be set.")
+        for nd in self.graph.nodes:
+            if nd.allow_cache_override:
+                nd.cache_dir = self.cache_dir
         # at this point Workflow is stateless so this should be fine
-        await submitter._run_workflow(self, rerun=rerun)
+        await submitter.expand_workflow(self, rerun=rerun)
 
     def set_output(self, connections):
         """
@@ -1014,7 +1204,6 @@ class Workflow(TaskBase):
         logger.info("Added %s to %s", self.output_spec, self)
 
     def _collect_outputs(self):
-        error = False
         output_klass = make_klass(self.output_spec)
         output = output_klass(**{f.name: None for f in attr.fields(output_klass)})
         # collecting outputs from tasks
@@ -1025,7 +1214,7 @@ class Workflow(TaskBase):
             try:
                 val_out = val.get_value(self)
                 output_wf[name] = val_out
-            except ValueError:
+            except (ValueError, AttributeError):
                 output_wf[name] = None
                 # checking if the tasks has predecessors that raises error
                 if isinstance(getattr(self, val.name)._errored, list):
@@ -1033,25 +1222,32 @@ class Workflow(TaskBase):
                         f"Tasks {getattr(self, val.name)._errored} raised an error"
                     )
                 else:
-                    raise ValueError(f"Task {val.name} raised an error")
+                    if isinstance(getattr(self, val.name).output_dir, list):
+                        err_file = [
+                            el / "_error.pklz"
+                            for el in getattr(self, val.name).output_dir
+                        ]
+                    else:
+                        err_file = getattr(self, val.name).output_dir / "_error.pklz"
+                    raise ValueError(
+                        f"Task {val.name} raised an error, full crash report is here: "
+                        f"{str(err_file)}"
+                    )
         return attr.evolve(output, **output_wf)
 
-    def create_dotfile(self, type="simple", export=None, name=None):
+    def create_dotfile(self, type="simple", export=None, name=None, output_dir=None):
         """creating a graph - dotfile and optionally exporting to other formats"""
+        outdir = output_dir if output_dir is not None else self.cache_dir
         if not name:
             name = f"graph_{self.name}"
         if type == "simple":
             for task in self.graph.nodes:
                 self.create_connections(task)
-            dotfile = self.graph.create_dotfile_simple(
-                outdir=self.output_dir, name=name
-            )
+            dotfile = self.graph.create_dotfile_simple(outdir=outdir, name=name)
         elif type == "nested":
             for task in self.graph.nodes:
                 self.create_connections(task)
-            dotfile = self.graph.create_dotfile_nested(
-                outdir=self.output_dir, name=name
-            )
+            dotfile = self.graph.create_dotfile_nested(outdir=outdir, name=name)
         elif type == "detailed":
             # create connections with detailed=True
             for task in self.graph.nodes:
@@ -1059,9 +1255,7 @@ class Workflow(TaskBase):
             # adding wf outputs
             for (wf_out, lf) in self._connections:
                 self.graph.add_edges_description((self.name, wf_out, lf.name, lf.field))
-            dotfile = self.graph.create_dotfile_detailed(
-                outdir=self.output_dir, name=name
-            )
+            dotfile = self.graph.create_dotfile_detailed(outdir=outdir, name=name)
         else:
             raise Exception(
                 f"type of the graph can be simple, detailed or nested, "
@@ -1079,6 +1273,32 @@ class Workflow(TaskBase):
                 formatted_dot.append(self.graph.export_graph(dotfile=dotfile, ext=ext))
             return dotfile, formatted_dot
 
+    def _connect_and_propagate_to_tasks(
+        self,
+        *,
+        propagate_rerun=False,
+        override_task_caches=False,
+    ):
+        """
+        Visit each node in the graph and create the connections.
+        Additionally checks if all tasks should be rerun.
+        """
+        for task in self.graph.nodes:
+            self.create_connections(task)
+            # if workflow has task_rerun=True and propagate_rerun=True,
+            # it should be passed to the tasks
+            if propagate_rerun:
+                task.task_rerun = True
+                # if the task is a wf, than the propagate_rerun should be also set
+                if is_workflow(task):
+                    task.propagate_rerun = True
+
+            # ported from Submitter.__call__
+            # TODO: no prepare state ?
+            if override_task_caches and task.allow_cache_override:
+                task.cache_dir = self.cache_dir
+            task.cache_locations = task._cache_locations + self.cache_locations
+
 
 def is_task(obj):
     """Check whether an object looks like a task."""
