@@ -4,10 +4,11 @@ import asyncio.subprocess as asp
 import attr
 import cloudpickle as cp
 from pathlib import Path
-from filelock import SoftFileLock
+from filelock import SoftFileLock, Timeout
 import os
 import sys
 from hashlib import sha256
+from uuid import uuid4
 import subprocess as sp
 import getpass
 import re
@@ -27,6 +28,8 @@ from .specs import (
     LazyField,
     MultiOutputObj,
     MultiInputObj,
+    MultiInputFile,
+    MultiOutputFile,
 )
 from .helpers_file import hash_file, hash_dir, copyfile, is_existing_file
 
@@ -60,6 +63,8 @@ def ensure_list(obj, tuple2list=False):
         return list(obj)
     elif isinstance(obj, list):
         return obj
+    elif isinstance(obj, LazyField):
+        return obj
     return [obj]
 
 
@@ -106,6 +111,8 @@ def load_result(checksum, cache_locations):
     """
     if not cache_locations:
         return None
+    # TODO: if there are issues with loading, we might need to
+    # TODO: sleep and repeat loads (after checking that there are no lock files!)
     for location in cache_locations:
         if (location / checksum).exists():
             result_file = location / checksum / "_result.pklz"
@@ -152,7 +159,7 @@ def save(task_path: Path, result=None, task=None, name_prefix=None):
 
 
 def copyfile_workflow(wf_path, result):
-    """ if file in the wf results, the file will be copied to the workflow directory"""
+    """if file in the wf results, the file will be copied to the workflow directory"""
     for field in attr_fields(result.output):
         value = getattr(result.output, field.name)
         # if the field is a path or it can contain a path _copyfile_single_value is run
@@ -168,7 +175,7 @@ def copyfile_workflow(wf_path, result):
 
 
 def _copyfile_single_value(wf_path, value):
-    """ checking a single value for files that need to be copied to the wf dir"""
+    """checking a single value for files that need to be copied to the wf dir"""
     if isinstance(value, (tuple, list)):
         return [_copyfile_single_value(wf_path, val) for val in value]
     elif isinstance(value, dict):
@@ -311,7 +318,15 @@ def custom_validator(instance, attribute, value):
         or value is None
         or attribute.name.startswith("_")  # e.g. _func
         or isinstance(value, LazyField)
-        or tp_attr in [ty.Any, inspect._empty, MultiOutputObj, MultiInputObj]
+        or tp_attr
+        in [
+            ty.Any,
+            inspect._empty,
+            MultiOutputObj,
+            MultiInputObj,
+            MultiOutputFile,
+            MultiInputFile,
+        ]
     ):
         check_type = False  # no checking of the type
     elif isinstance(tp_attr, type) or tp_attr in [File, Directory]:
@@ -339,7 +354,8 @@ def custom_validator(instance, attribute, value):
                 tp = {"key": tp_k, "val": tp_v}
         else:
             warnings.warn(
-                f"no type check for {attribute.name} field, no type check implemented for value {value} and type {tp_attr}"
+                f"no type check for {attribute.name} field, "
+                f"no type check implemented for value {value} and type {tp_attr}"
             )
             check_type = False
 
@@ -354,7 +370,7 @@ def custom_validator(instance, attribute, value):
 
 
 def _type_validator(instance, attribute, value, tp, cont_type):
-    """ creating a customized type validator,
+    """creating a customized type validator,
     uses validator.deep_iterable/mapping if the field is a container
     (i.e. ty.List or ty.Dict),
     it also tries to guess when the value is a list due to the splitter
@@ -386,7 +402,8 @@ def _type_validator(instance, attribute, value, tp, cont_type):
         )(instance, attribute, value)
     else:
         raise Exception(
-            f"container type of {attribute.name} should be None, list, dict or ty.Union, and not {cont_type}"
+            f"container type of {attribute.name} should be None, list, dict or ty.Union, "
+            f"and not {cont_type}"
         )
 
 
@@ -406,9 +423,9 @@ def _types_updates(tp_list, name):
 
 
 def _single_type_update(tp, name, simplify=False):
-    """ updating a single type with other related types - e.g. adding bytes for str
-        if simplify is True, than changing typing.List to list etc.
-        (assuming that I validate only one depth, so have to simplify at some point)
+    """updating a single type with other related types - e.g. adding bytes for str
+    if simplify is True, than changing typing.List to list etc.
+    (assuming that I validate only one depth, so have to simplify at some point)
     """
     if isinstance(tp, type) or tp in [File, Directory]:
         if tp is str:
@@ -435,13 +452,14 @@ def _single_type_update(tp, name, simplify=False):
             return None
     else:
         warnings.warn(
-            f"no type check for {name} field, type check not implemented for type - {tp}, consider using simplify=True"
+            f"no type check for {name} field, type check not implemented for type - {tp}, "
+            f"consider using simplify=True"
         )
         return None
 
 
 def _check_special_type(tp, name):
-    """checking if the type is a container: ty.List, ty.Dict or ty.Union """
+    """checking if the type is a container: ty.List, ty.Dict or ty.Union"""
     if sys.version_info.minor >= 8:
         return ty.get_origin(tp), ty.get_args(tp)
     else:
@@ -462,9 +480,9 @@ def _check_special_type(tp, name):
 
 
 def _allowed_values_validator(instance, attribute, value):
-    """ checking if the values is in allowed_values"""
+    """checking if the values is in allowed_values"""
     allowed = attribute.metadata["allowed_values"]
-    if value is attr.NOTHING:
+    if value is attr.NOTHING or isinstance(value, LazyField):
         pass
     elif value not in allowed:
         raise ValueError(
@@ -637,12 +655,17 @@ def get_open_loop():
     """
     if os.name == "nt":
         loop = asyncio.ProactorEventLoop()  # for subprocess' pipes on Windows
-        asyncio.set_event_loop(loop)
     else:
-        loop = asyncio.get_event_loop()
-        if loop.is_closed():
+        try:
+            loop = asyncio.get_event_loop()
+        # in case RuntimeError: There is no current event loop in thread 'MainThread'
+        except RuntimeError:
             loop = asyncio.new_event_loop()
             asyncio.set_event_loop(loop)
+        else:
+            if loop.is_closed():
+                loop = asyncio.new_event_loop()
+                asyncio.set_event_loop(loop)
     return loop
 
 
@@ -651,14 +674,16 @@ def hash_function(obj):
     return sha256(str(obj).encode()).hexdigest()
 
 
-def hash_value(value, tp=None, metadata=None):
+def hash_value(value, tp=None, metadata=None, precalculated=None):
     """calculating hash or returning values recursively"""
     if metadata is None:
         metadata = {}
-    if isinstance(value, (tuple, list)):
-        return [hash_value(el, tp, metadata) for el in value]
+    if isinstance(value, (tuple, list, set)):
+        return [hash_value(el, tp, metadata, precalculated) for el in value]
     elif isinstance(value, dict):
-        dict_hash = {k: hash_value(v, tp, metadata) for (k, v) in value.items()}
+        dict_hash = {
+            k: hash_value(v, tp, metadata, precalculated) for (k, v) in value.items()
+        }
         # returning a sorted object
         return [list(el) for el in sorted(dict_hash.items(), key=lambda x: x[0])]
     else:  # not a container
@@ -667,13 +692,18 @@ def hash_value(value, tp=None, metadata=None):
             and is_existing_file(value)
             and "container_path" not in metadata
         ):
-            return hash_file(value)
+            return hash_file(value, precalculated=precalculated)
         elif (
             (tp is File or "pydra.engine.specs.Directory" in str(tp))
             and is_existing_file(value)
             and "container_path" not in metadata
         ):
-            return hash_dir(value)
+            return hash_dir(value, precalculated=precalculated)
+        elif type(value).__module__ == "numpy":  # numpy objects
+            return [
+                hash_value(el, tp, metadata, precalculated)
+                for el in ensure_list(value.tolist())
+            ]
         else:
             return value
 
@@ -700,7 +730,7 @@ def output_from_inputfields(output_spec, input_spec):
                 field_name = fld.metadata["output_field_name"]
             else:
                 field_name = fld.name
-            # not adding if the field already in teh output_spec
+            # not adding if the field already in the output_spec
             if field_name not in current_output_spec_names:
                 # TODO: should probably remove some of the keys
                 new_fields.append(
@@ -742,12 +772,12 @@ def load_and_run(
     task_pkl, ind=None, rerun=False, submitter=None, plugin=None, **kwargs
 ):
     """
-     loading a task from a pickle file, settings proper input
-     and running the task
-     """
+    loading a task from a pickle file, settings proper input
+    and running the task
+    """
     try:
         task = load_task(task_pkl=task_pkl, ind=ind)
-    except Exception as excinfo:
+    except Exception:
         if task_pkl.parent.exists():
             etype, eval, etr = sys.exc_info()
             traceback = format_exception(etype, eval, etr)
@@ -762,10 +792,11 @@ def load_and_run(
     except Exception as excinfo:
         # creating result and error files if missing
         errorfile = task.output_dir / "_error.pklz"
-        if not resultfile.exists():
+        if not errorfile.exists():  # not sure if this is needed
             etype, eval, etr = sys.exc_info()
             traceback = format_exception(etype, eval, etr)
             errorfile = record_error(task.output_dir, error=traceback)
+        if not resultfile.exists():  # not sure if this is needed
             result = Result(output=None, runtime=None, errored=True)
             save(task.output_dir, result=result)
         raise type(excinfo)(
@@ -785,7 +816,7 @@ async def load_and_run_async(task_pkl, ind=None, submitter=None, rerun=False, **
 
 
 def load_task(task_pkl, ind=None):
-    """ loading a task from a pickle file, settings proper input for the specific ind"""
+    """loading a task from a pickle file, settings proper input for the specific ind"""
     if isinstance(task_pkl, str):
         task_pkl = Path(task_pkl)
     task = cp.loads(task_pkl.read_bytes())
@@ -793,57 +824,59 @@ def load_task(task_pkl, ind=None):
         _, inputs_dict = task.get_input_el(ind)
         task.inputs = attr.evolve(task.inputs, **inputs_dict)
         task.state = None
+        # resetting uid for task
+        task._uid = uuid4().hex
     return task
 
 
-def position_adjustment(pos_args):
+def position_sort(args):
     """
-    sorting elements with the first element - position,
-    the negative positions should go to the end of the list
-    everything that has no position (i.e. it's None),
-    should go between elements with positive positions an with negative pos.
-    Returns a list of sorted args.
+    Sort objects by position, following Python indexing conventions.
+
+    Ordering is positive positions, lowest to highest, followed by unspecified
+    positions (``None``) and negative positions, lowest to highest.
+
+    >>> position_sort([(None, "d"), (-3, "e"), (2, "b"), (-2, "f"), (5, "c"), (1, "a")])
+    ['a', 'b', 'c', 'd', 'e', 'f']
+
+    Parameters
+    ----------
+    args : list of (int/None, object) tuples
+
+    Returns
+    -------
+    list of objects
     """
-    # sorting all elements of the command
-    try:
-        pos_args.sort()
-    except TypeError:  # if some positions are None
-        pos_args_none = []
-        pos_args_int = []
-        for el in pos_args:
-            if el[0] is None:
-                pos_args_none.append(el)
-            else:
-                pos_args_int.append(el)
-            pos_args_int.sort()
-        last_el = pos_args_int[-1][0]
-        for el_none in pos_args_none:
-            last_el += 1
-            pos_args_int.append((last_el, el_none[1]))
-        pos_args = pos_args_int
+    import bisect
 
-    # if args available, they should be moved at the of the list
-    while pos_args[0][0] < 0:
-        pos_args.append(pos_args.pop(0))
+    pos, none, neg = [], [], []
+    for entry in args:
+        position = entry[0]
+        if position is None:
+            # Take existing order
+            none.append(entry[1])
+        elif position < 0:
+            # Sort negatives while collecting
+            bisect.insort(neg, entry)
+        else:
+            # Sort positives while collecting
+            bisect.insort(pos, entry)
 
-    # dropping the position index
-    cmd_args = []
-    for el in pos_args:
-        cmd_args += el[1]
-
-    return cmd_args
+    return [arg for _, arg in pos] + none + [arg for _, arg in neg]
 
 
 def argstr_formatting(argstr, inputs, value_updates=None):
-    """ formatting argstr that have form {field_name},
+    """formatting argstr that have form {field_name},
     using values from inputs and updating with value_update if provided
     """
-    inputs_dict = attr.asdict(inputs)
+    inputs_dict = attr.asdict(inputs, recurse=False)
     # if there is a value that has to be updated (e.g. single value from a list)
     if value_updates:
         inputs_dict.update(value_updates)
     # getting all fields that should be formatted, i.e. {field_name}, ...
-    inp_fields = re.findall("{\w+}", argstr)
+    inp_fields = re.findall(r"{\w+}", argstr)
+    inp_fields_float = re.findall(r"{\w+:[0-9.]+f}", argstr)
+    inp_fields += [re.sub(":[0-9.]+f", "", el) for el in inp_fields_float]
     val_dict = {}
     for fld in inp_fields:
         fld_name = fld[1:-1]  # extracting the name form {field_name}
@@ -865,3 +898,29 @@ def argstr_formatting(argstr, inputs, value_updates=None):
         .strip()
     )
     return argstr_formatted
+
+
+class PydraFileLock:
+    """Wrapper for filelock's SoftFileLock that makes it work with asyncio."""
+
+    def __init__(self, lockfile):
+        self.lockfile = lockfile
+        self.timeout = 0.1
+
+    async def __aenter__(self):
+        lock = SoftFileLock(self.lockfile)
+        acquired_lock = False
+        while not acquired_lock:
+            try:
+                lock.acquire(timeout=0)
+                acquired_lock = True
+            except Timeout:
+                await asyncio.sleep(self.timeout)
+                if self.timeout <= 2:
+                    self.timeout = self.timeout * 2
+        self.lock = lock
+        return self
+
+    async def __aexit__(self, exc_type, exc_value, traceback):
+        self.lock.release()
+        return None
