@@ -2,7 +2,6 @@
 
 import attr
 import itertools
-from functools import reduce
 from copy import deepcopy
 import logging
 from .helpers import ensure_list
@@ -56,81 +55,91 @@ def _ordering(
     if type(el) is tuple:
         # checking if the splitter dont contain splitter from previous nodes
         # i.e. has str "_NA", etc.
-        if type(el[0]) is str and el[0].startswith("_"):
-            node_nm = el[0][1:]
-            if node_nm not in other_states and state_fields:
-                raise PydraStateError(
-                    "can't ask for splitter from {}, other nodes that are connected: {}".format(
-                        node_nm, other_states.keys()
+        if len(el) == 1:
+            # treats .split(("x",)) like .split("x")
+            el = el[0]
+            _ordering(el, i, output_splitter, current_sign, other_states, state_fields)
+        else:
+            if type(el[0]) is str and el[0].startswith("_"):
+                node_nm = el[0][1:]
+                if node_nm not in other_states and state_fields:
+                    raise PydraStateError(
+                        "can't ask for splitter from {}, other nodes that are connected: {}".format(
+                            node_nm, other_states.keys()
+                        )
                     )
-                )
-            elif state_fields:
-                splitter_mod = add_name_splitter(
-                    splitter=other_states[node_nm][0].splitter_final, name=node_nm
-                )
-                el = (splitter_mod, el[1])
-                if other_states[node_nm][0].other_states:
-                    other_states.update(other_states[node_nm][0].other_states)
-        if type(el[1]) is str and el[1].startswith("_"):
-            node_nm = el[1][1:]
-            if node_nm not in other_states and state_fields:
-                raise PydraStateError(
-                    "can't ask for splitter from {}, other nodes that are connected: {}".format(
-                        node_nm, other_states.keys()
+                elif state_fields:
+                    splitter_mod = add_name_splitter(
+                        splitter=other_states[node_nm][0].splitter_final, name=node_nm
                     )
-                )
-            elif state_fields:
-                splitter_mod = add_name_splitter(
-                    splitter=other_states[node_nm][0].splitter_final, name=node_nm
-                )
-                el = (el[0], splitter_mod)
-                if other_states[node_nm][0].other_states:
-                    other_states.update(other_states[node_nm][0].other_states)
-        _iterate_list(
-            el,
-            ".",
-            other_states,
-            output_splitter=output_splitter,
-            state_fields=state_fields,
-        )
+                    el = (splitter_mod, el[1])
+                    if other_states[node_nm][0].other_states:
+                        other_states.update(other_states[node_nm][0].other_states)
+            if type(el[1]) is str and el[1].startswith("_"):
+                node_nm = el[1][1:]
+                if node_nm not in other_states and state_fields:
+                    raise PydraStateError(
+                        "can't ask for splitter from {}, other nodes that are connected: {}".format(
+                            node_nm, other_states.keys()
+                        )
+                    )
+                elif state_fields:
+                    splitter_mod = add_name_splitter(
+                        splitter=other_states[node_nm][0].splitter_final, name=node_nm
+                    )
+                    el = (el[0], splitter_mod)
+                    if other_states[node_nm][0].other_states:
+                        other_states.update(other_states[node_nm][0].other_states)
+            _iterate_list(
+                el,
+                ".",
+                other_states,
+                output_splitter=output_splitter,
+                state_fields=state_fields,
+            )
     elif type(el) is list:
-        if type(el[0]) is str and el[0].startswith("_"):
-            node_nm = el[0][1:]
-            if node_nm not in other_states and state_fields:
-                raise PydraStateError(
-                    "can't ask for splitter from {}, other nodes that are connected: {}".format(
-                        node_nm, other_states.keys()
+        if len(el) == 1:
+            # treats .split(["x"]) like .split("x")
+            el = el[0]
+            _ordering(el, i, output_splitter, current_sign, other_states, state_fields)
+        else:
+            if type(el[0]) is str and el[0].startswith("_"):
+                node_nm = el[0][1:]
+                if node_nm not in other_states and state_fields:
+                    raise PydraStateError(
+                        "can't ask for splitter from {}, other nodes that are connected: {}".format(
+                            node_nm, other_states.keys()
+                        )
                     )
-                )
-            elif state_fields:
-                splitter_mod = add_name_splitter(
-                    splitter=other_states[node_nm][0].splitter_final, name=node_nm
-                )
-                el[0] = splitter_mod
-                if other_states[node_nm][0].other_states:
-                    other_states.update(other_states[node_nm][0].other_states)
-        if type(el[1]) is str and el[1].startswith("_"):
-            node_nm = el[1][1:]
-            if node_nm not in other_states and state_fields:
-                raise PydraStateError(
-                    "can't ask for splitter from {}, other nodes that are connected: {}".format(
-                        node_nm, other_states.keys()
+                elif state_fields:
+                    splitter_mod = add_name_splitter(
+                        splitter=other_states[node_nm][0].splitter_final, name=node_nm
                     )
-                )
-            elif state_fields:
-                splitter_mod = add_name_splitter(
-                    splitter=other_states[node_nm][0].splitter_final, name=node_nm
-                )
-                el[1] = splitter_mod
-                if other_states[node_nm][0].other_states:
-                    other_states.update(other_states[node_nm][0].other_states)
-        _iterate_list(
-            el,
-            "*",
-            other_states,
-            output_splitter=output_splitter,
-            state_fields=state_fields,
-        )
+                    el[0] = splitter_mod
+                    if other_states[node_nm][0].other_states:
+                        other_states.update(other_states[node_nm][0].other_states)
+            if type(el[1]) is str and el[1].startswith("_"):
+                node_nm = el[1][1:]
+                if node_nm not in other_states and state_fields:
+                    raise PydraStateError(
+                        "can't ask for splitter from {}, other nodes that are connected: {}".format(
+                            node_nm, other_states.keys()
+                        )
+                    )
+                elif state_fields:
+                    splitter_mod = add_name_splitter(
+                        splitter=other_states[node_nm][0].splitter_final, name=node_nm
+                    )
+                    el[1] = splitter_mod
+                    if other_states[node_nm][0].other_states:
+                        other_states.update(other_states[node_nm][0].other_states)
+            _iterate_list(
+                el,
+                "*",
+                other_states,
+                output_splitter=output_splitter,
+                state_fields=state_fields,
+            )
     elif type(el) is str:
         if el.startswith("_"):
             node_nm = el[1:]
@@ -307,7 +316,7 @@ def rpn2splitter(splitter_rpn):
 
 
 def add_name_combiner(combiner, name):
-    """ adding a node's name to each field from the combiner"""
+    """adding a node's name to each field from the combiner"""
     combiner_changed = []
     for comb in combiner:
         if "." not in comb:
@@ -318,7 +327,7 @@ def add_name_combiner(combiner, name):
 
 
 def add_name_splitter(splitter, name):
-    """ adding a node's name to each field from the splitter"""
+    """adding a node's name to each field from the splitter"""
     if isinstance(splitter, str):
         return _add_name([splitter], name)[0]
     elif isinstance(splitter, list):
@@ -329,7 +338,7 @@ def add_name_splitter(splitter, name):
 
 
 def _add_name(mlist, name):
-    """ adding anem to each element from the list"""
+    """adding anem to each element from the list"""
     for i, elem in enumerate(mlist):
         if isinstance(elem, str):
             if "." in elem or elem.startswith("_"):
@@ -345,9 +354,6 @@ def _add_name(mlist, name):
     return mlist
 
 
-op = {".": zip, "*": itertools.product}
-
-
 def flatten(vals, cur_depth=0, max_depth=None):
     """Flatten a list of values."""
     if max_depth is None:
@@ -371,7 +377,7 @@ def iter_splits(iterable, keys):
 
 
 def input_shape(inp, cont_dim=1):
-    """Get input shape, depends on the container dimension, if not specify it is assumed to be 1 """
+    """Get input shape, depends on the container dimension, if not specify it is assumed to be 1"""
     # TODO: have to be changed for inner splitter (sometimes different length)
     cont_dim -= 1
     shape = [len(inp)]
@@ -392,178 +398,9 @@ def input_shape(inp, cont_dim=1):
     return tuple(shape)
 
 
-def splits(splitter_rpn, inputs, inner_inputs=None, cont_dim=None):
-    """
-    Splits input variable as specified by splitter
-
-    Parameters
-    ----------
-    splitter_rpn : list
-        splitter in RPN notation
-    inputs: dict
-        input variables
-    inner_inputs: dict, optional
-        inner input specification
-    cont_dim: dict, optional
-        container dimension for input variable, specifies how nested is the intput,
-        if not specified 1 will be used for all inputs (so will not be flatten)
-
-
-    Returns
-    -------
-    splitter : list
-        each element contains indices for inputs
-    keys: list
-        names of input variables
-
-    """
-
-    stack = []
-    keys = []
-    if cont_dim is None:
-        cont_dim = {}
-    # analysing states from connected tasks if inner_inputs
-    if inner_inputs:
-        previous_states_ind = {
-            f"_{v.name}": (v.ind_l_final, v.keys_final) for v in inner_inputs.values()
-        }
-        inner_inputs = {k: v for k, v in inner_inputs.items() if k in splitter_rpn}
-    else:
-        previous_states_ind = {}
-        inner_inputs = {}
-
-    # when splitter is a single element (no operators)
-    if len(splitter_rpn) == 1:
-        op_single = splitter_rpn[0]
-        return _single_op_splits(
-            op_single, inputs, inner_inputs, previous_states_ind, cont_dim=cont_dim
-        )
-
-    terms = {}
-    trm_val = {}
-    trm_str = {}
-    shape = {}
-    # iterating splitter_rpn
-    for token in splitter_rpn:
-        if token not in [".", "*"]:  # token is one of the input var
-            # adding variable to the stack
-            stack.append(token)
-        else:
-            # removing Right and Left var from the stack
-            terms["R"] = stack.pop()
-            terms["L"] = stack.pop()
-            # checking if terms are strings, shapes, etc.
-            for lr in ["L", "R"]:
-                term = terms[lr]
-                if isinstance(term, str):
-                    if term.startswith("_"):
-                        trm_val[lr] = previous_states_ind[term][0]
-                        shape[lr] = (len(trm_val[lr]),)
-                    else:
-                        if term in cont_dim:
-                            shape[lr] = input_shape(
-                                inputs[term], cont_dim=cont_dim[term]
-                            )
-                        else:
-                            shape[lr] = input_shape(inputs[term])
-                        trm_val[lr] = range(reduce(lambda x, y: x * y, shape[lr]))
-                    trm_str[lr] = True
-                else:
-                    trm_val[lr], shape[lr] = term
-                    trm_str[lr] = False
-
-            # checking shapes and creating newshape
-            if token == ".":
-                if shape["L"] != shape["R"]:
-                    raise ValueError(
-                        "Operands {} and {} do not have same shape.".format(
-                            terms["R"], terms["L"]
-                        )
-                    )
-                newshape = shape["R"]
-            if token == "*":
-                # TODO: pomyslec
-                newshape = tuple(list(shape["L"]) + list(shape["R"]))
-
-            # creating list with keys
-            new_keys = {}
-            for lr in ["R", "L"]:
-                if trm_str[lr]:
-                    if terms[lr].startswith("_"):
-                        new_keys[lr] = previous_states_ind[terms[lr]][1]
-                    else:
-                        new_keys[lr] = [terms[lr]]
-
-            if trm_str["L"] and trm_str["R"]:
-                # TODO: check why i'm using this order
-                keys = keys + new_keys["L"] + new_keys["R"]
-            elif trm_str["L"]:
-                keys = new_keys["L"] + keys
-            elif trm_str["R"]:
-                keys = keys + new_keys["R"]
-
-            newtrm_val = {}
-            for lr in ["R", "L"]:
-                # TODO: rewrite once I have more tests
-                if isinstance(terms[lr], str) and terms[lr] in inner_inputs:
-                    # TODO: have to be changed if differ length
-                    inner_len = [shape[lr][-1]] * reduce(
-                        lambda x, y: x * y, shape[lr][:-1]
-                    )
-                    # this come from the previous node
-                    outer_ind = inner_inputs[terms[lr]].ind_l
-                    trmval_out = itertools.chain.from_iterable(
-                        itertools.repeat(x, n) for x, n in zip(outer_ind, inner_len)
-                    )
-                    newtrm_val[lr] = op["."](trmval_out, trm_val[lr])
-                    keys = (
-                        keys[: keys.index(terms[lr])]
-                        + inner_inputs[terms[lr]].keys_final
-                        + keys[keys.index(terms[lr]) :]
-                    )
-                else:
-                    newtrm_val[lr] = trm_val[lr]
-
-            pushval = (op[token](newtrm_val["L"], newtrm_val["R"]), newshape)
-            stack.append(pushval)
-
-    val = stack.pop()
-    if isinstance(val, tuple):
-        val = val[0]
-    return val, keys
-
-
-def _single_op_splits(
-    op_single, inputs, inner_inputs, previous_states_ind, cont_dim=None
-):
-    """ splits function if splitter is a singleton"""
-    if op_single.startswith("_"):
-        return (previous_states_ind[op_single][0], previous_states_ind[op_single][1])
-    if cont_dim is None:
-        cont_dim = {}
-    shape = input_shape(inputs[op_single], cont_dim=cont_dim.get(op_single, 1))
-    trmval = range(reduce(lambda x, y: x * y, shape))
-    if op_single in inner_inputs:
-        # TODO: have to be changed if differ length
-        inner_len = [shape[-1]] * reduce(lambda x, y: x * y, shape[:-1])
-        # this come from the previous node
-        outer_ind = inner_inputs[op_single].ind_l
-        op_out = itertools.chain.from_iterable(
-            itertools.repeat(x, n) for x, n in zip(outer_ind, inner_len)
-        )
-        res = op["."](op_out, trmval)
-        val = res
-        keys = inner_inputs[op_single].keys_final + [op_single]
-        return val, keys
-    else:
-        val = op["*"](trmval)
-        keys = [op_single]
-        return val, keys
-
-
 def splits_groups(splitter_rpn, combiner=None, inner_inputs=None):
-    """ splits inputs to groups (axes) and creates stacks for these groups
-        This is used to specify which input can be combined.
+    """splits inputs to groups (axes) and creates stacks for these groups
+    This is used to specify which input can be combined.
     """
     if not splitter_rpn:
         return [], {}, [], []
@@ -696,7 +533,7 @@ def splits_groups(splitter_rpn, combiner=None, inner_inputs=None):
 
 
 def _single_op_splits_groups(op_single, combiner, inner_inputs, groups):
-    """ splits_groups function if splitter is a singleton"""
+    """splits_groups function if splitter is a singleton"""
     if op_single in inner_inputs:
         # TODO: have to be changed if differ length
         # TODO: i think I don't want to add here from left part
@@ -712,9 +549,8 @@ def _single_op_splits_groups(op_single, combiner, inner_inputs, groups):
         else:
             # TODO: probably not needed, should be already check by st.combiner_validation
             raise PydraStateError(
-                "all fields from the combiner have to be in splitter_rpn: {}, but combiner: {} is set".format(
-                    [op_single], combiner
-                )
+                f"all fields from the combiner have to be in splitter_rpn: {[op_single]}, "
+                f"but combiner: {combiner} is set"
             )
     else:
         return keys, groups, groups_stack, []
@@ -747,15 +583,25 @@ def combine_final_groups(combiner, groups, groups_stack, keys):
                     "first".format(comb, groups_stack[-1])
                 )
     groups_final = {inp: gr for (inp, gr) in groups.items() if inp not in combiner_all}
-    gr_final = list(set(groups_final.values()))
+    gr_final = set()
+    for el in groups_final.values():
+        gr_final.update(ensure_list(el))
+    gr_final = list(gr_final)
     map_gr_nr = {nr: i for (i, nr) in enumerate(sorted(gr_final))}
-    groups_final = {inp: map_gr_nr[gr] for (inp, gr) in groups_final.items()}
+    groups_final_map = {}
+    for (inp, gr) in groups_final.items():
+        if isinstance(gr, int):
+            groups_final_map[inp] = map_gr_nr[gr]
+        elif isinstance(gr, list):
+            groups_final_map[inp] = [map_gr_nr[el] for el in gr]
+        else:
+            raise Exception("gr should be an int or a list, something wrong")
     for i, groups_l in enumerate(groups_stack_final):
         groups_stack_final[i] = [map_gr_nr[gr] for gr in groups_l]
 
     keys_final = [key for key in keys if key not in combiner_all]
     # TODO: not sure if I have to calculate and return keys, groups, groups_stack
-    return keys_final, groups_final, groups_stack_final, combiner_all
+    return keys_final, groups_final_map, groups_stack_final, combiner_all
 
 
 def map_splits(split_iter, inputs, cont_dim=None):
@@ -772,7 +618,9 @@ def map_splits(split_iter, inputs, cont_dim=None):
 def inputs_types_to_dict(name, inputs):
     """Convert type.Inputs to dictionary."""
     # dj: any better option?
-    input_names = [field for field in attr.asdict(inputs) if field != "_func"]
+    input_names = [
+        field for field in attr.asdict(inputs, recurse=False) if field != "_func"
+    ]
     inputs_dict = {}
     for field in input_names:
         inputs_dict[f"{name}.{field}"] = getattr(inputs, field)
