@@ -4,12 +4,21 @@ from pathlib import Path
 import typing as ty
 import inspect
 import re
+from glob import glob
 
 from .helpers_file import template_update_single
 
 
-def attr_fields(x):
-    return x.__attrs_attrs__
+def attr_fields(spec, exclude_names=()):
+    return [field for field in spec.__attrs_attrs__ if field.name not in exclude_names]
+
+
+def attr_fields_dict(spec, exclude_names=()):
+    return {
+        field.name: field
+        for field in spec.__attrs_attrs__
+        if field.name not in exclude_names
+    }
 
 
 class File:
@@ -27,7 +36,10 @@ class MultiInputObj:
     def converter(cls, value):
         from .helpers import ensure_list
 
-        return ensure_list(value)
+        if value == attr.NOTHING:
+            return value
+        else:
+            return ensure_list(value)
 
 
 class MultiOutputObj:
@@ -41,6 +53,14 @@ class MultiOutputObj:
             return value
 
 
+class MultiInputFile(MultiInputObj):
+    """A ty.List[File] object, converter changes a single file path to a list"""
+
+
+class MultiOutputFile(MultiOutputObj):
+    """A ty.List[File] object, converter changes an 1-el list to the single value"""
+
+
 @attr.s(auto_attribs=True, kw_only=True)
 class SpecInfo:
     """Base data structure for metadata of specifications."""
@@ -58,22 +78,36 @@ class SpecInfo:
 class BaseSpec:
     """The base dataclass specs for all inputs and outputs."""
 
+    def __attrs_post_init__(self):
+        self.files_hash = {
+            field.name: {}
+            for field in attr_fields(
+                self, exclude_names=("_graph_checksums", "bindings", "files_hash")
+            )
+            if field.metadata.get("output_file_template") is None
+        }
+
     def __setattr__(self, name, value):
         """changing settatr, so the converter and validator is run
         if input is set after __init__
         """
-        if inspect.stack()[1][3] == "__init__":  # or name.startswith("_"):
+        if inspect.stack()[1][3] == "__init__" or name in [
+            "inp_hash",
+            "changed",
+            "files_hash",
+        ]:
             super().__setattr__(name, value)
         else:
             tp = attr.fields_dict(self.__class__)[name].type
             # if the type has a converter, e.g., MultiInputObj
             if hasattr(tp, "converter"):
                 value = tp.converter(value)
+            self.files_hash[name] = {}
             super().__setattr__(name, value)
             # validate all fields that have set a validator
             attr.validate(self)
 
-    def collect_additional_outputs(self, inputs, output_dir):
+    def collect_additional_outputs(self, inputs, output_dir, outputs):
         """Get additional outputs."""
         return {}
 
@@ -83,22 +117,25 @@ class BaseSpec:
         from .helpers import hash_value, hash_function
 
         inp_dict = {}
-        for field in attr_fields(self):
-            if field.name in ["_graph_checksums", "bindings"] or field.metadata.get(
-                "output_file_template"
-            ):
+        for field in attr_fields(
+            self, exclude_names=("_graph_checksums", "bindings", "files_hash")
+        ):
+            if field.metadata.get("output_file_template"):
                 continue
-            # removing values that are notset from hash calculation
+            # removing values that are not set from hash calculation
             if getattr(self, field.name) is attr.NOTHING:
                 continue
+            value = getattr(self, field.name)
             inp_dict[field.name] = hash_value(
-                value=getattr(self, field.name), tp=field.type, metadata=field.metadata
+                value=value,
+                tp=field.type,
+                metadata=field.metadata,
+                precalculated=self.files_hash[field.name],
             )
         inp_hash = hash_function(inp_dict)
         if hasattr(self, "_graph_checksums"):
-            return hash_function((inp_hash, self._graph_checksums))
-        else:
-            return inp_hash
+            inp_hash = hash_function((inp_hash, self._graph_checksums))
+        return inp_hash
 
     def retrieve_values(self, wf, state_index=None):
         """Get values contained by this spec."""
@@ -126,9 +163,21 @@ class BaseSpec:
             # checking if the mandatory field is provided
             if getattr(self, fld.name) is attr.NOTHING:
                 if mdata.get("mandatory"):
-                    raise AttributeError(
-                        f"{fld.name} is mandatory, but no value provided"
-                    )
+                    # checking if the mandatory field is provided elsewhere in the xor list
+                    in_exclusion_list = mdata.get("xor") is not None
+                    alreday_populated = in_exclusion_list and [
+                        getattr(self, el)
+                        for el in mdata["xor"]
+                        if (getattr(self, el) is not attr.NOTHING)
+                    ]
+                    if (
+                        alreday_populated
+                    ):  # another input satisfies mandatory attribute via xor condition
+                        continue
+                    else:
+                        raise AttributeError(
+                            f"{fld.name} is mandatory, but no value provided"
+                        )
                 else:
                     continue
             names.append(fld.name)
@@ -145,19 +194,36 @@ class BaseSpec:
                     # will check after adding all fields to names
                     require_to_check[fld.name] = mdata["requires"]
 
-            if fld.type is File:
-                self._file_check_n_bindings(fld)
+            if (
+                fld.type in [File, Directory]
+                or "pydra.engine.specs.File" in str(fld.type)
+                or "pydra.engine.specs.Directory" in str(fld.type)
+            ):
+                self._file_check(fld)
 
         for nm, required in require_to_check.items():
             required_notfound = [el for el in required if el not in names]
             if required_notfound:
                 raise AttributeError(f"{nm} requires {required_notfound}")
 
-    def _file_check_n_bindings(self, field):
-        """for tasks without container, this is simple check if the file exists"""
-        file = Path(getattr(self, field.name))
-        if not file.exists():
-            raise AttributeError(f"the file from the {field.name} input does not exist")
+    def _file_check(self, field):
+        """checking if the file exists"""
+        if isinstance(getattr(self, field.name), list):
+            # if value is a list and type is a list of Files/Directory, checking all elements
+            if field.type in [ty.List[File], ty.List[Directory]]:
+                for el in getattr(self, field.name):
+                    file = Path(el)
+                    if not file.exists() and field.type in [File, Directory]:
+                        raise FileNotFoundError(
+                            f"the file {file} from the {field.name} input does not exist"
+                        )
+        else:
+            file = Path(getattr(self, field.name))
+            # error should be raised only if the type is strictly File or Directory
+            if not file.exists() and field.type in [File, Directory]:
+                raise FileNotFoundError(
+                    f"the file {file} from the {field.name} input does not exist"
+                )
 
     def check_metadata(self):
         """Check contained metadata."""
@@ -194,7 +260,7 @@ class Result:
         if state["output"] is not None:
             fields = tuple((el.name, el.type) for el in attr_fields(state["output"]))
             state["output_spec"] = (state["output"].__class__.__name__, fields)
-            state["output"] = attr.asdict(state["output"])
+            state["output"] = attr.asdict(state["output"], recurse=False)
         return state
 
     def __setstate__(self, state):
@@ -216,7 +282,7 @@ class Result:
             Name of field in LazyField object
         """
         if field_name == "all_":
-            return attr.asdict(self.output)
+            return attr.asdict(self.output, recurse=False)
         else:
             return getattr(self.output, field_name)
 
@@ -270,11 +336,7 @@ class FunctionSpec(BaseSpec):
             "xor",
             "sep",
         }
-        # special inputs, don't have to follow rules for standard inputs
-        special_input = ["_func", "_graph_checksums"]
-
-        fields = [fld for fld in attr_fields(self) if fld.name not in special_input]
-        for fld in fields:
+        for fld in attr_fields(self, exclude_names=("_func", "_graph_checksums")):
             mdata = fld.metadata
             # checking keys from metadata
             if set(mdata.keys()) - supported_keys:
@@ -350,12 +412,9 @@ class ShellSpec(BaseSpec):
             "keep_extension",
             "xor",
             "sep",
+            "formatter",
         }
-        # special inputs, don't have to follow rules for standard inputs
-        special_input = ["_func", "_graph_checksums"]
-
-        fields = [fld for fld in attr_fields(self) if fld.name not in special_input]
-        for fld in fields:
+        for fld in attr_fields(self, exclude_names=("_func", "_graph_checksums")):
             mdata = fld.metadata
             # checking keys from metadata
             if set(mdata.keys()) - supported_keys:
@@ -395,31 +454,78 @@ class ShellOutSpec:
     stderr: ty.Union[File, str]
     """The process' standard input."""
 
-    def collect_additional_outputs(self, inputs, output_dir):
+    def collect_additional_outputs(self, inputs, output_dir, outputs):
         """Collect additional outputs from shelltask output_spec."""
         additional_out = {}
-        for fld in attr_fields(self):
-            if fld.name not in ["return_code", "stdout", "stderr"]:
-                if fld.type is File:
-                    # assuming that field should have either default or metadata, but not both
-                    if (
-                        fld.default is None or fld.default == attr.NOTHING
-                    ) and not fld.metadata:  # TODO: is it right?
-                        raise AttributeError(
-                            "File has to have default value or metadata"
-                        )
-                    elif not fld.default == attr.NOTHING:
-                        additional_out[fld.name] = self._field_defaultvalue(
-                            fld, output_dir
-                        )
-                    elif fld.metadata:
-                        additional_out[fld.name] = self._field_metadata(
-                            fld, inputs, output_dir
-                        )
-                else:
-                    raise Exception("not implemented (collect_additional_output)")
+        for fld in attr_fields(self, exclude_names=("return_code", "stdout", "stderr")):
+            if fld.type not in [
+                File,
+                MultiOutputFile,
+                Directory,
+                Path,
+                int,
+                float,
+                bool,
+                str,
+                list,
+            ]:
+                raise Exception(
+                    f"Support for {fld.type} type, required for {fld.name} in {self}, "
+                    "has not been implemented in collect_additional_output"
+                )
+            # assuming that field should have either default or metadata, but not both
+            input_value = getattr(inputs, fld.name, attr.NOTHING)
+            if input_value is not attr.NOTHING:
+                if fld.type in (File, MultiOutputFile, Directory, Path):
+                    input_value = Path(input_value).absolute()
+                additional_out[fld.name] = input_value
+            elif (
+                fld.default is None or fld.default == attr.NOTHING
+            ) and not fld.metadata:  # TODO: is it right?
+                raise AttributeError("File has to have default value or metadata")
+            elif fld.default != attr.NOTHING:
+                additional_out[fld.name] = self._field_defaultvalue(fld, output_dir)
+            elif fld.metadata:
+                if (
+                    fld.type in [int, float, bool, str, list]
+                    and "callable" not in fld.metadata
+                ):
+                    raise AttributeError(
+                        f"{fld.type} has to have a callable in metadata"
+                    )
+                additional_out[fld.name] = self._field_metadata(
+                    fld, inputs, output_dir, outputs
+                )
         return additional_out
 
+    def generated_output_names(self, inputs, output_dir):
+        """Returns a list of all outputs that will be generated by the task.
+        Takes into account the task input and the requires list for the output fields.
+        TODO: should be in all Output specs?
+        """
+        # checking the input (if all mandatory fields are provided, etc.)
+        inputs.check_fields_input_spec()
+        output_names = ["return_code", "stdout", "stderr"]
+        for fld in attr_fields(self, exclude_names=("return_code", "stdout", "stderr")):
+            if fld.type not in [File, MultiOutputFile, Directory]:
+                raise Exception("not implemented (collect_additional_output)")
+            # assuming that field should have either default or metadata, but not both
+            if (
+                fld.default in (None, attr.NOTHING) and not fld.metadata
+            ):  # TODO: is it right?
+                raise AttributeError("File has to have default value or metadata")
+            elif fld.default != attr.NOTHING:
+                output_names.append(fld.name)
+            elif (
+                fld.metadata
+                and self._field_metadata(
+                    fld, inputs, output_dir, outputs=None, check_existance=False
+                )
+                != attr.NOTHING
+            ):
+                output_names.append(fld.name)
+        return output_names
+
     def _field_defaultvalue(self, fld, output_dir):
         """Collect output file if the default value specified."""
         if not isinstance(fld.default, (str, Path)):
@@ -433,14 +539,13 @@ class ShellOutSpec:
             default = Path(default)
 
         default = output_dir / default
-
-        if "*" not in default.name:
+        if "*" not in str(default):
             if default.exists():
                 return default
             else:
                 raise AttributeError(f"file {default} does not exist")
         else:
-            all_files = list(Path(default.parent).expanduser().glob(default.name))
+            all_files = [Path(el) for el in glob(str(default.expanduser()))]
             if len(all_files) > 1:
                 return all_files
             elif len(all_files) == 1:
@@ -448,7 +553,9 @@ class ShellOutSpec:
             else:
                 raise AttributeError(f"no file matches {default.name}")
 
-    def _field_metadata(self, fld, inputs, output_dir):
+    def _field_metadata(
+        self, fld, inputs, output_dir, outputs=None, check_existance=True
+    ):
         """Collect output file if metadata specified."""
         if self._check_requires(fld, inputs) is False:
             return attr.NOTHING
@@ -459,19 +566,62 @@ class ShellOutSpec:
         # if the field is set in input_spec with output_file_template,
         # than the field already should have value
         elif "output_file_template" in fld.metadata:
-            inputs_templ = attr.asdict(inputs)
             value = template_update_single(
-                fld, inputs_templ, output_dir=output_dir, spec_type="output"
+                fld, inputs=inputs, output_dir=output_dir, spec_type="output"
             )
-            return Path(value)
+
+            if fld.type is MultiOutputFile and type(value) is list:
+                # TODO: how to deal with mandatory list outputs
+                ret = []
+                for val in value:
+                    val = Path(val)
+                    if check_existance and not val.exists():
+                        ret.append(attr.NOTHING)
+                    else:
+                        ret.append(val)
+                return ret
+            else:
+                val = Path(value)
+                # checking if the file exists
+                if check_existance and not val.exists():
+                    # if mandatory raise exception
+                    if "mandatory" in fld.metadata:
+                        if fld.metadata["mandatory"]:
+                            raise Exception(
+                                f"mandatory output for variable {fld.name} does not exist"
+                            )
+                    return attr.NOTHING
+                return val
         elif "callable" in fld.metadata:
-            return fld.metadata["callable"](fld.name, output_dir)
+            call_args = inspect.getfullargspec(fld.metadata["callable"])
+            call_args_val = {}
+            for argnm in call_args.args:
+                if argnm == "field":
+                    call_args_val[argnm] = fld
+                elif argnm == "output_dir":
+                    call_args_val[argnm] = output_dir
+                elif argnm == "inputs":
+                    call_args_val[argnm] = inputs
+                elif argnm == "stdout":
+                    call_args_val[argnm] = outputs["stdout"]
+                elif argnm == "stderr":
+                    call_args_val[argnm] = outputs["stderr"]
+                else:
+                    try:
+                        call_args_val[argnm] = getattr(inputs, argnm)
+                    except AttributeError:
+                        raise AttributeError(
+                            f"arguments of the callable function from {fld.name} "
+                            f"has to be in inputs or be field or output_dir, "
+                            f"but {argnm} is used"
+                        )
+            return fld.metadata["callable"](**call_args_val)
         else:
             raise Exception("(_field_metadata) is not a current valid metadata key.")
 
     def _check_requires(self, fld, inputs):
-        """ checking if all fields from the requires and template are set in the input
-            if requires is a list of list, checking if at least one list has all elements set
+        """checking if all fields from the requires and template are set in the input
+        if requires is a list of list, checking if at least one list has all elements set
         """
         from .helpers import ensure_list
 
@@ -492,9 +642,14 @@ class ShellOutSpec:
             field_required_OR = [[]]
 
         for field_required in field_required_OR:
-            # if the output has output_file_template field, adding all input fields from the template to requires
+            # if the output has output_file_template field,
+            # adding all input fields from the template to requires
             if "output_file_template" in fld.metadata:
-                inp_fields = re.findall("{\w+}", fld.metadata["output_file_template"])
+                template = fld.metadata["output_file_template"]
+                # if a template is a function it has to be run first with the inputs as the only arg
+                if callable(template):
+                    template = template(inputs)
+                inp_fields = re.findall(r"{\w+}", template)
                 field_required += [
                     el[1:-1] for el in inp_fields if el[1:-1] not in field_required
                 ]
@@ -552,36 +707,6 @@ class ContainerSpec(ShellSpec):
     container_xargs: ty.Optional[ty.List[str]] = attr.ib(
         default=None, metadata={"help_string": "todo"}
     )
-    """Execution arguments to run the image."""
-    bindings: ty.Optional[
-        ty.List[
-            ty.Tuple[
-                Path,  # local path
-                Path,  # container path
-                ty.Optional[str],  # mount mode
-            ]
-        ]
-    ] = attr.ib(default=None, metadata={"help_string": "bindings"})
-    """Mount points to be bound into the container."""
-
-    def _file_check_n_bindings(self, field):
-        file = Path(getattr(self, field.name))
-        if field.metadata.get("container_path"):
-            # if the path is in a container the input should be treated as a str (hash as a str)
-            # field.type = "str"
-            # setattr(self, field.name, str(file))
-            pass
-        # if this is a local path, checking if the path exists
-        elif file.exists():
-            if self.bindings is None:
-                self.bindings = []
-            self.bindings.append((file.parent, f"/pydra_inp_{field.name}", "ro"))
-        else:
-            raise Exception(
-                f"the file from {field.name} input does not exist, "
-                f"if the file comes from the container, "
-                f"use field.metadata['container_path']=True"
-            )
 
 
 @attr.s(auto_attribs=True, kw_only=True)
