@@ -38,17 +38,19 @@ Implement processing nodes.
       <https://colab.research.google.com/drive/1RRV1gHbGJs49qQB1q1d5tQEycVRtuhw6>`__
 
 """
+import platform
+import re
 import attr
 import cloudpickle as cp
 import inspect
 import typing as ty
+import shlex
 from pathlib import Path
 import warnings
 
 from .core import TaskBase, is_lazy
 from ..utils.messenger import AuditFlag
 from .specs import (
-    File,
     BaseSpec,
     SpecInfo,
     ShellSpec,
@@ -57,11 +59,13 @@ from .specs import (
     DockerSpec,
     SingularitySpec,
     attr_fields,
+    File,
+    Directory,
 )
 from .helpers import (
     ensure_list,
     execute,
-    position_adjustment,
+    position_sort,
     argstr_formatting,
     output_from_inputfields,
 )
@@ -78,6 +82,7 @@ class FunctionTask(TaskBase):
         cache_dir=None,
         cache_locations=None,
         input_spec: ty.Optional[ty.Union[SpecInfo, BaseSpec]] = None,
+        cont_dim=None,
         messenger_args=None,
         messengers=None,
         name=None,
@@ -100,6 +105,9 @@ class FunctionTask(TaskBase):
             List of alternative cache locations.
         input_spec : :obj:`pydra.engine.specs.SpecInfo`
             Specification of inputs.
+        cont_dim : :obj:`dict`, or `None`
+            Container dimensions for input fields,
+            if any of the container should be treated as a container
         messenger_args :
             TODO
         messengers :
@@ -141,6 +149,7 @@ class FunctionTask(TaskBase):
         super().__init__(
             name,
             inputs=kwargs,
+            cont_dim=cont_dim,
             audit_flags=audit_flags,
             messengers=messengers,
             messenger_args=messenger_args,
@@ -149,69 +158,51 @@ class FunctionTask(TaskBase):
             rerun=rerun,
         )
         if output_spec is None:
-            if "return" not in func.__annotations__:
-                output_spec = SpecInfo(
-                    name="Output", fields=[("out", ty.Any)], bases=(BaseSpec,)
-                )
-            else:
+            name = "Output"
+            fields = [("out", ty.Any)]
+            if "return" in func.__annotations__:
                 return_info = func.__annotations__["return"]
                 # e.g. python annotation: fun() -> ty.NamedTuple("Output", [("out", float)])
                 # or pydra decorator: @pydra.mark.annotate({"return": ty.NamedTuple(...)})
-                if hasattr(return_info, "__name__") and hasattr(
-                    return_info, "__annotations__"
+                if hasattr(return_info, "__name__") and getattr(
+                    return_info, "__annotations__", None
                 ):
-                    output_spec = SpecInfo(
-                        name=return_info.__name__,
-                        fields=list(return_info.__annotations__.items()),
-                        bases=(BaseSpec,),
-                    )
+                    name = return_info.__name__
+                    fields = list(return_info.__annotations__.items())
                 # e.g. python annotation: fun() -> {"out": int}
                 # or pydra decorator: @pydra.mark.annotate({"return": {"out": int}})
                 elif isinstance(return_info, dict):
-                    output_spec = SpecInfo(
-                        name="Output",
-                        fields=list(return_info.items()),
-                        bases=(BaseSpec,),
-                    )
+                    fields = list(return_info.items())
                 # e.g. python annotation: fun() -> (int, int)
                 # or pydra decorator: @pydra.mark.annotate({"return": (int, int)})
                 elif isinstance(return_info, tuple):
-                    output_spec = SpecInfo(
-                        name="Output",
-                        fields=[
-                            ("out{}".format(n + 1), t)
-                            for n, t in enumerate(return_info)
-                        ],
-                        bases=(BaseSpec,),
-                    )
+                    fields = [(f"out{i}", t) for i, t in enumerate(return_info, 1)]
                 # e.g. python annotation: fun() -> int
                 # or pydra decorator: @pydra.mark.annotate({"return": int})
                 else:
-                    output_spec = SpecInfo(
-                        name="Output", fields=[("out", return_info)], bases=(BaseSpec,)
-                    )
+                    fields = [("out", return_info)]
+            output_spec = SpecInfo(name=name, fields=fields, bases=(BaseSpec,))
+
         self.output_spec = output_spec
 
     def _run_task(self):
-        inputs = attr.asdict(self.inputs)
+        inputs = attr.asdict(self.inputs, recurse=False)
         del inputs["_func"]
         self.output_ = None
         output = cp.loads(self.inputs._func)(**inputs)
         output_names = [el[0] for el in self.output_spec.fields]
         if output is None:
             self.output_ = {nm: None for nm in output_names}
+        elif len(output_names) == 1:
+            # if only one element in the fields, everything should be returned together
+            self.output_ = {output_names[0]: output}
+        elif isinstance(output, tuple) and len(output_names) == len(output):
+            self.output_ = dict(zip(output_names, output))
         else:
-            if len(output_names) == 1:
-                # if only one element in the fields, everything should be returned together
-                self.output_ = {output_names[0]: output}
-            else:
-                if isinstance(output, tuple) and len(output_names) == len(output):
-                    self.output_ = dict(zip(output_names, output))
-                else:
-                    raise Exception(
-                        f"expected {len(self.output_spec.fields)} elements, "
-                        f"but {output} were returned"
-                    )
+            raise RuntimeError(
+                f"expected {len(self.output_spec.fields)} elements, "
+                f"but {output} were returned"
+            )
 
 
 class ShellCommandTask(TaskBase):
@@ -224,19 +215,23 @@ class ShellCommandTask(TaskBase):
         if not container_info:
             return super().__new__(cls)
 
-        if len(container_info) == 3:
-            type_cont, image, bind = container_info
-        elif len(container_info) == 2:
-            type_cont, image, bind = container_info + (None,)
+        if len(container_info) == 2:
+            type_cont, image = container_info
         else:
             raise Exception(
-                f"container_info has to have 2 or 3 elements, but {container_info} provided"
+                f"container_info has to have 2 elements, but {container_info} provided"
             )
 
         if type_cont == "docker":
-            return DockerTask(image=image, bindings=bind, *args, **kwargs)
+            # changing base class of spec if user defined
+            if "input_spec" in kwargs:
+                kwargs["input_spec"].bases = (DockerSpec,)
+            return DockerTask(image=image, *args, **kwargs)
         elif type_cont == "singularity":
-            return SingularityTask(image=image, bindings=bind, *args, **kwargs)
+            # changing base class of spec if user defined
+            if "input_spec" in kwargs:
+                kwargs["input_spec"].bases = (SingularitySpec,)
+            return SingularityTask(image=image, *args, **kwargs)
         else:
             raise Exception(
                 f"first element of container_info has to be "
@@ -248,6 +243,7 @@ class ShellCommandTask(TaskBase):
         audit_flags: AuditFlag = AuditFlag.NONE,
         cache_dir=None,
         input_spec: ty.Optional[SpecInfo] = None,
+        cont_dim=None,
         messenger_args=None,
         messengers=None,
         name=None,
@@ -267,6 +263,9 @@ class ShellCommandTask(TaskBase):
             Cache directory
         input_spec : :obj:`pydra.engine.specs.SpecInfo`
             Specification of inputs.
+        cont_dim : :obj:`dict`, or `None`
+            Container dimensions for input fields,
+            if any of the container should be treated as a container
         messenger_args :
             TODO
         messengers :
@@ -280,6 +279,10 @@ class ShellCommandTask(TaskBase):
 
         """
 
+        # using default name for task if no name provided
+        if name is None:
+            name = "ShellTask_noname"
+
         # using provided spec, class attribute or setting the default SpecInfo
         self.input_spec = (
             input_spec
@@ -299,12 +302,14 @@ class ShellCommandTask(TaskBase):
                     kwargs[special_inp] = getattr(self, special_inp)
                 elif kwargs[special_inp] != getattr(self, special_inp):
                     warnings.warn(
-                        f"you are changing the executable from {getattr(self, special_inp)} to {kwargs[special_inp]}"
+                        f"you are changing the executable from {getattr(self, special_inp)} "
+                        f"to {kwargs[special_inp]}"
                     )
 
         super().__init__(
             name=name,
             inputs=kwargs,
+            cont_dim=cont_dim,
             audit_flags=audit_flags,
             messengers=messengers,
             messenger_args=messenger_args,
@@ -315,132 +320,138 @@ class ShellCommandTask(TaskBase):
 
     @property
     def command_args(self):
-        """Get command line arguments, returns a list if task has a state"""
+        """Get command line arguments"""
         if is_lazy(self.inputs):
             raise Exception("can't return cmdline, self.inputs has LazyFields")
         if self.state:
-            command_args_list = []
-            self.state.prepare_states(self.inputs)
-            for ii, el in enumerate(self.state.states_ind):
-                command_args_list.append(self._command_args_single(el, ind=ii))
-            return command_args_list
-        else:
-            return self._command_args_single(self.inputs)
+            raise NotImplementedError
+
+        modified_inputs = template_update(self.inputs, output_dir=self.output_dir)
+        if modified_inputs is not None:
+            self.inputs = attr.evolve(self.inputs, **modified_inputs)
 
-    def _command_args_single(self, state_ind, ind=None):
-        """Get command line arguments for a single state"""
         pos_args = []  # list for (position, command arg)
         self._positions_provided = []
-        for f in attr_fields(self.inputs):
-            # these inputs will eb used in container_args
-            if isinstance(self, ContainerTask) and f.name in [
-                "container",
-                "image",
-                "container_xargs",
-                "bindings",
-            ]:
+        for field in attr_fields(
+            self.inputs,
+            exclude_names=("container", "image", "container_xargs"),
+        ):
+            name, meta = field.name, field.metadata
+            if (
+                getattr(self.inputs, name) is attr.NOTHING
+                and not meta.get("readonly")
+                and not meta.get("formatter")
+            ):
                 continue
-            elif f.name == "executable":
-                pos_args.append(
-                    self._command_shelltask_executable(
-                        field=f, state_ind=state_ind, ind=ind
-                    )
-                )
-            elif f.name == "args":
-                pos_val = self._command_shelltask_args(
-                    field=f, state_ind=state_ind, ind=ind
-                )
+            if name == "executable":
+                pos_args.append(self._command_shelltask_executable(field))
+            elif name == "args":
+                pos_val = self._command_shelltask_args(field)
                 if pos_val:
                     pos_args.append(pos_val)
             else:
-                pos_val = self._command_pos_args(field=f, state_ind=state_ind, ind=ind)
+                pos_val = self._command_pos_args(field)
                 if pos_val:
                     pos_args.append(pos_val)
 
-        # sorted elements of the command
-        cmd_args = position_adjustment(pos_args)
-        return cmd_args
+        # Sort command and arguments by position
+        cmd_args = position_sort(pos_args)
+        # pos_args values are each a list of arguments, so concatenate lists after sorting
+        return sum(cmd_args, [])
 
-    def _field_value(self, field, state_ind, ind, check_file=False):
+    def _field_value(self, field, check_file=False):
         """
         Checking value of the specific field, if value is not set, None is returned.
-        If state_ind and ind, taking a specific element of the field.
-        If check_file is True, checking if field is a a local file
-        and settings bindings if needed.
+        check_file has no effect, but subclasses can use it to validate or modify
+        filenames.
         """
-        name = f"{self.name}.{field.name}"
-        if self.state and name in state_ind:
-            value = getattr(self.inputs, field.name)[state_ind[name]]
-        else:
-            value = getattr(self.inputs, field.name)
-        if value is attr.NOTHING or value is None:
-            return None
-        if check_file:
-            if is_local_file(field):
-                value = str(value)
-                # changing path to the cpath (the directory should be mounted)
-                if getattr(self, "bind_paths", None):
-                    lpath = Path(value)
-                    cdir = self.bind_paths(ind=ind)[lpath.parent][0]
-                    cpath = cdir.joinpath(lpath.name)
-                    value = str(cpath)
+        value = getattr(self.inputs, field.name)
+        if value == attr.NOTHING:
+            value = None
         return value
 
-    def _command_shelltask_executable(self, field, state_ind, ind):
+    def _command_shelltask_executable(self, field):
         """Returining position and value for executable ShellTask input"""
         pos = 0  # executable should be the first el. of the command
-        value = self._field_value(field=field, state_ind=state_ind, ind=ind)
+        value = self._field_value(field)
         if value is None:
-            raise Exception("executable has to be set")
+            raise ValueError("executable has to be set")
         return pos, ensure_list(value, tuple2list=True)
 
-    def _command_shelltask_args(self, field, state_ind, ind):
+    def _command_shelltask_args(self, field):
         """Returining position and value for args ShellTask input"""
         pos = -1  # assuming that args is the last el. of the command
-        value = self._field_value(
-            field=field, state_ind=state_ind, ind=ind, check_file=True
-        )
+        value = self._field_value(field, check_file=True)
         if value is None:
             return None
         else:
             return pos, ensure_list(value, tuple2list=True)
 
-    def _command_pos_args(self, field, state_ind, ind):
+    def _command_pos_args(self, field):
         """
         Checking all additional input fields, setting pos to None, if position not set.
         Creating a list with additional parts of the command that comes from
         the specific field.
         """
         argstr = field.metadata.get("argstr", None)
-        if argstr is None:
-            # assuming that input that has no arstr is not used in the command
+        formatter = field.metadata.get("formatter", None)
+        if argstr is None and formatter is None:
+            # assuming that input that has no arstr is not used in the command,
+            # or a formatter is not provided too.
             return None
         pos = field.metadata.get("position", None)
-        if pos is None:
-            # position will be set at the end
-            pass
-        elif not isinstance(pos, int):
-            raise Exception(f"position should be an integer, but {pos} given")
-        elif pos == 0:
-            raise Exception(f"position can't be 0")
-        elif pos < 0:  # position -1 is for args
-            pos = pos - 1
-        # checking if the position is not already used
-        elif pos in self._positions_provided:
-            raise Exception(
-                f"{field.name} can't have provided position, {pos} is already used"
-            )
-        self._positions_provided.append(pos)
-        value = self._field_value(
-            field=field, state_ind=state_ind, ind=ind, check_file=True
-        )
+        if pos is not None:
+            if not isinstance(pos, int):
+                raise Exception(f"position should be an integer, but {pos} given")
+            # checking if the position is not already used
+            if pos in self._positions_provided:
+                raise Exception(
+                    f"{field.name} can't have provided position, {pos} is already used"
+                )
+
+            self._positions_provided.append(pos)
+
+            # Shift non-negatives up to allow executable to be 0
+            # Shift negatives down to allow args to be -1
+            pos += 1 if pos >= 0 else -1
+
+        value = self._field_value(field, check_file=True)
         if field.metadata.get("readonly", False) and value is not None:
             raise Exception(f"{field.name} is read only, the value can't be provided")
-        elif value is None and not field.metadata.get("readonly", False):
+        elif (
+            value is None
+            and not field.metadata.get("readonly", False)
+            and formatter is None
+        ):
             return None
 
+        inputs_dict = attr.asdict(self.inputs, recurse=False)
+
         cmd_add = []
-        if field.type is bool:
+        # formatter that creates a custom command argument
+        # it can thake the value of the filed, all inputs, or the value of other fields.
+        if "formatter" in field.metadata:
+            call_args = inspect.getfullargspec(field.metadata["formatter"])
+            call_args_val = {}
+            for argnm in call_args.args:
+                if argnm == "field":
+                    call_args_val[argnm] = value
+                elif argnm == "inputs":
+                    call_args_val[argnm] = inputs_dict
+                else:
+                    if argnm in inputs_dict:
+                        call_args_val[argnm] = inputs_dict[argnm]
+                    else:
+                        raise AttributeError(
+                            f"arguments of the formatter function from {field.name} "
+                            f"has to be in inputs or be field or output_dir, "
+                            f"but {argnm} is used"
+                        )
+            cmd_el_str = field.metadata["formatter"](**call_args_val)
+            cmd_el_str = cmd_el_str.strip().replace("  ", " ")
+            if cmd_el_str != "":
+                cmd_add += split_cmd(cmd_el_str)
+        elif field.type is bool:
             # if value is simply True the original argstr is used,
             # if False, nothing is added to the command
             if value is True:
@@ -456,7 +467,7 @@ class ShellCommandTask(TaskBase):
                         argstr_f = argstr_formatting(
                             argstr, self.inputs, value_updates={field.name: val}
                         )
-                        argstr_formatted_l.append(argstr_f)
+                        argstr_formatted_l.append(f" {argstr_f}")
                     cmd_el_str = sep.join(argstr_formatted_l)
                 else:  # argstr has a simple form, e.g. "-f", or "--f"
                     cmd_el_str = sep.join([f" {argstr} {val}" for val in value])
@@ -468,47 +479,42 @@ class ShellCommandTask(TaskBase):
                     value = cmd_el_str
                 # if argstr has a more complex form, with "{input_field}"
                 if "{" in argstr and "}" in argstr:
-                    cmd_el_str = argstr_formatting(argstr, self.inputs)
+                    cmd_el_str = argstr.replace(f"{{{field.name}}}", str(value))
+                    cmd_el_str = argstr_formatting(cmd_el_str, self.inputs)
                 else:  # argstr has a simple form, e.g. "-f", or "--f"
                     if value:
                         cmd_el_str = f"{argstr} {value}"
                     else:
                         cmd_el_str = ""
-            # removing double spacing
-            cmd_el_str = cmd_el_str.strip().replace("  ", " ")
             if cmd_el_str:
-                cmd_add += cmd_el_str.split(" ")
+                cmd_add += split_cmd(cmd_el_str)
         return pos, cmd_add
 
     @property
     def cmdline(self):
-        """ Get the actual command line that will be submitted
-            Returns a list if the task has a state.
+        """Get the actual command line that will be submitted
+        Returns a list if the task has a state.
         """
         if is_lazy(self.inputs):
             raise Exception("can't return cmdline, self.inputs has LazyFields")
         # checking the inputs fields before returning the command line
         self.inputs.check_fields_input_spec()
-        orig_inputs = attr.asdict(self.inputs)
-        modified_inputs = template_update(self.inputs, output_dir=self.output_dir)
-        if modified_inputs is not None:
-            self.inputs = attr.evolve(self.inputs, **modified_inputs)
+        if self.state:
+            raise NotImplementedError
         if isinstance(self, ContainerTask):
-            if self.state:
-                cmdline = []
-                for con, com in zip(self.container_args, self.command_args):
-                    cmdline.append(" ".join(con + com))
-            else:
-                cmdline = " ".join(self.container_args + self.command_args)
+            command_args = self.container_args + self.command_args
         else:
-            if self.state:
-                cmdline = []
-                for el in self.command_args:
-                    cmdline.append(" ".join(el))
+            command_args = self.command_args
+        # Skip the executable, which can be a multi-part command, e.g. 'docker run'.
+        cmdline = command_args[0]
+        for arg in command_args[1:]:
+            # If there are spaces in the arg and it is not enclosed by matching
+            # quotes, add quotes to escape the space. Not sure if this should
+            # be expanded to include other special characters apart from spaces
+            if " " in arg:
+                cmdline += " '" + arg + "'"
             else:
-                cmdline = " ".join(self.command_args)
-
-        self.inputs = attr.evolve(self.inputs, **orig_inputs)
+                cmdline += " " + arg
         return cmdline
 
     def _run_task(self):
@@ -524,10 +530,12 @@ class ShellCommandTask(TaskBase):
             values = execute(args, strip=self.strip)
             self.output_ = dict(zip(keys, values))
             if self.output_["return_code"]:
+                msg = f"Error running '{self.name}' task with {args}:"
                 if self.output_["stderr"]:
-                    raise RuntimeError(self.output_["stderr"])
-                else:
-                    raise RuntimeError(self.output_["stdout"])
+                    msg += "\n\nstderr:\n" + self.output_["stderr"]
+                if self.output_["stdout"]:
+                    msg += "\n\nstdout:\n" + self.output_["stdout"]
+                raise RuntimeError(msg)
 
 
 class ContainerTask(ShellCommandTask):
@@ -575,6 +583,7 @@ class ContainerTask(ShellCommandTask):
         if input_spec is None:
             input_spec = SpecInfo(name="Inputs", fields=[], bases=(ContainerSpec,))
         self.output_cpath = Path(output_cpath)
+        self.bindings = {}
         super().__init__(
             name=name,
             input_spec=input_spec,
@@ -588,6 +597,21 @@ class ContainerTask(ShellCommandTask):
             **kwargs,
         )
 
+    def _field_value(self, field, check_file=False):
+        """
+        Checking value of the specific field, if value is not set, None is returned.
+        If check_file is True, checking if field is a a local file
+        and settings bindings if needed.
+        """
+        value = super()._field_value(field)
+        if value and check_file and is_local_file(field):
+            # changing path to the cpath (the directory should be mounted)
+            lpath = Path(str(value))
+            cdir = self.bind_paths()[lpath.parent][0]
+            cpath = cdir.joinpath(lpath.name)
+            value = str(cpath)
+        return value
+
     def container_check(self, container_type):
         """Get container-specific CLI arguments."""
         if self.inputs.container is None:
@@ -599,47 +623,61 @@ class ContainerTask(ShellCommandTask):
         if self.inputs.image is attr.NOTHING:
             raise AttributeError("Container image is not specified")
 
-    def bind_paths(self, ind=None):
-        """Return bound mount points: ``dict(lpath: (cpath, mode))``."""
-        bind_paths = {}
-        output_dir_cpath = None
-        if self.inputs.bindings is None:
-            self.inputs.bindings = []
-        if ind is None:
-            output_dir = self.output_dir
-        else:
-            output_dir = self.output_dir[ind]
-        for binding in self.inputs.bindings:
-            if len(binding) == 3:
-                lpath, cpath, mode = binding
-            elif len(binding) == 2:
-                lpath, cpath, mode = binding + ["rw"]
-            else:
-                raise Exception(
-                    f"binding should have length 2, 3, or 4, it has {len(binding)}"
-                )
-            if Path(lpath) == output_dir:
-                output_dir_cpath = cpath
-            if mode is None:
-                mode = "rw"  # default
-            bind_paths[Path(lpath)] = (Path(cpath), mode)
-        # output_dir is added to the bindings if not part of self.inputs.bindings
-        if not output_dir_cpath:
-            bind_paths[output_dir] = (self.output_cpath, "rw")
-        return bind_paths
+    def bind_paths(self):
+        """Get bound mount points
 
-    def binds(self, opt, ind=None):
+        Returns
+        -------
+        mount points: dict
+            mapping from local path to tuple of container path + mode
+        """
+        self._check_inputs()
+        return {**self.bindings, **{self.output_dir: (self.output_cpath, "rw")}}
+
+    def binds(self, opt):
         """
         Specify mounts to bind from local filesystems to container and working directory.
 
-        Uses py:meth:`binds_paths`
+        Uses py:meth:`bind_paths`
 
         """
         bargs = []
-        for (key, val) in self.bind_paths(ind).items():
-            bargs.extend([opt, f"{key}:{val[0]}:{val[1]}"])
+        for lpath, (cpath, mode) in self.bind_paths().items():
+            bargs.extend([opt, f"{lpath}:{cpath}:{mode}"])
         return bargs
 
+    def _check_inputs(self):
+        fields = attr_fields(self.inputs)
+        for fld in fields:
+            if (
+                fld.type in [File, Directory]
+                or "pydra.engine.specs.File" in str(fld.type)
+                or "pydra.engine.specs.Directory" in str(fld.type)
+            ):
+
+                if fld.name == "image":
+                    continue
+                file = Path(getattr(self.inputs, fld.name))
+                if fld.metadata.get("container_path"):
+                    # if the path is in a container the input should be treated as a str (hash as a str)
+                    # field.type = "str"
+                    # setattr(self, field.name, str(file))
+                    pass
+                # if this is a local path, checking if the path exists
+                # TODO: if copyfile, ro -> rw
+                elif file.exists():  # is it ok if two inputs have the same parent?
+                    self.bindings[Path(file.parent)] = (
+                        Path(f"/pydra_inp_{fld.name}"),
+                        "ro",
+                    )
+                # error should be raised only if the type is strictly File or Directory
+                elif fld.type in [File, Directory]:
+                    raise FileNotFoundError(
+                        f"the file {file} from {fld.name} input does not exist, "
+                        f"if the file comes from the container, "
+                        f"use field.metadata['container_path']=True"
+                    )
+
 
 class DockerTask(ContainerTask):
     """Extend shell command task for containerized execution with the Docker Engine."""
@@ -648,7 +686,7 @@ class DockerTask(ContainerTask):
 
     def __init__(
         self,
-        name,
+        name=None,
         audit_flags: AuditFlag = AuditFlag.NONE,
         cache_dir=None,
         input_spec: ty.Optional[SpecInfo] = None,
@@ -688,7 +726,6 @@ class DockerTask(ContainerTask):
         if not self.init:
             if input_spec is None:
                 input_spec = SpecInfo(name="Inputs", fields=[], bases=(DockerSpec,))
-
             super().__init__(
                 name=name,
                 input_spec=input_spec,
@@ -712,31 +749,15 @@ class DockerTask(ContainerTask):
             raise Exception("can't return container_args, self.inputs has LazyFields")
         self.container_check("docker")
         if self.state:
-            self.state.prepare_states(self.inputs)
-            cargs_list = []
-            for ii, el in enumerate(self.state.states_ind):
-                if f"{self.name}.image" in el:
-                    cargs_list.append(
-                        self._container_args_single(
-                            self.inputs.image[el[f"{self.name}.image"]], ind=ii
-                        )
-                    )
-                else:
-                    cargs_list.append(
-                        self._container_args_single(self.inputs.image, ind=ii)
-                    )
-            return cargs_list
-        else:
-            return self._container_args_single(self.inputs.image)
+            raise NotImplementedError
 
-    def _container_args_single(self, image, ind=None):
         cargs = ["docker", "run"]
         if self.inputs.container_xargs is not None:
             cargs.extend(self.inputs.container_xargs)
 
-        cargs.extend(self.binds("-v", ind))
+        cargs.extend(self.binds("-v"))
         cargs.extend(["-w", str(self.output_cpath)])
-        cargs.append(image)
+        cargs.append(self.inputs.image)
 
         return cargs
 
@@ -748,6 +769,7 @@ class SingularityTask(ContainerTask):
 
     def __init__(
         self,
+        name=None,
         audit_flags: AuditFlag = AuditFlag.NONE,
         cache_dir=None,
         input_spec: ty.Optional[SpecInfo] = None,
@@ -787,6 +809,7 @@ class SingularityTask(ContainerTask):
                     name="Inputs", fields=[], bases=(SingularitySpec,)
                 )
             super().__init__(
+                name=name,
                 input_spec=input_spec,
                 output_spec=output_spec,
                 audit_flags=audit_flags,
@@ -806,31 +829,40 @@ class SingularityTask(ContainerTask):
             raise Exception("can't return container_args, self.inputs has LazyFields")
         self.container_check("singularity")
         if self.state:
-            self.state.prepare_states(self.inputs)
-            cargs_list = []
-            for ii, el in enumerate(self.state.states_ind):
-                if f"{self.name}.image" in el:
-                    cargs_list.append(
-                        self._container_args_single(
-                            self.inputs.image[el[f"{self.name}.image"]], ind=ii
-                        )
-                    )
-                else:
-                    cargs_list.append(
-                        self._container_args_single(self.inputs.image, ind=ii)
-                    )
-            return cargs_list
-        else:
-            return self._container_args_single(self.inputs.image)
+            raise NotImplementedError
 
-    def _container_args_single(self, image, ind=None):
         cargs = ["singularity", "exec"]
 
         if self.inputs.container_xargs is not None:
             cargs.extend(self.inputs.container_xargs)
-        cargs.append(image)
 
-        # insert bindings before image
-        idx = len(cargs) - 1
-        cargs[idx:-1] = self.binds("-B", ind)
+        cargs.extend(self.binds("-B"))
+        cargs.extend(["--pwd", str(self.output_cpath)])
+        cargs.append(self.inputs.image)
         return cargs
+
+
+def split_cmd(cmd: str):
+    """Splits a shell command line into separate arguments respecting quotes
+
+    Parameters
+    ----------
+    cmd : str
+        Command line string or part thereof
+
+    Returns
+    -------
+    str
+        the command line string split into process args
+    """
+    # Check whether running on posix or windows system
+    on_posix = platform.system() != "Windows"
+    args = shlex.split(cmd, posix=on_posix)
+    cmd_args = []
+    for arg in args:
+        match = re.match("('|\")(.*)\\1$", arg)
+        if match:
+            cmd_args.append(match.group(2))
+        else:
+            cmd_args.append(arg)
+    return cmd_args
