@@ -1,16 +1,28 @@
 import attr
 import typing as ty
+from pathlib import Path
 import pytest
 
 from ..task import ShellCommandTask
-from ..specs import ShellOutSpec, ShellSpec, SpecInfo, File
+from ..specs import (
+    ShellOutSpec,
+    ShellSpec,
+    SpecInfo,
+    File,
+    MultiInputObj,
+    MultiInputFile,
+    MultiOutputFile,
+)
 from .utils import use_validator
+from ..core import Workflow
+from ..submitter import Submitter
 
 
 def test_shell_cmd_execargs_1():
     # separate command into exec + args
     shelly = ShellCommandTask(executable="executable", args="arg")
     assert shelly.cmdline == "executable arg"
+    assert shelly.name == "ShellTask_noname"
 
 
 def test_shell_cmd_execargs_2():
@@ -20,7 +32,7 @@ def test_shell_cmd_execargs_2():
 
 
 def test_shell_cmd_inputs_1():
-    """ additional input with provided position """
+    """additional input with provided position"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -42,7 +54,7 @@ def test_shell_cmd_inputs_1():
 
 
 def test_shell_cmd_inputs_1a():
-    """ additional input without provided position """
+    """additional input without provided position"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -54,12 +66,12 @@ def test_shell_cmd_inputs_1a():
     shelly = ShellCommandTask(
         executable="executable", args="arg", inpA="inpNone1", input_spec=my_input_spec
     )
-    # inp1 should be firt one after executable
+    # inp1 should be the first one after executable
     assert shelly.cmdline == "executable inpNone1 arg"
 
 
 def test_shell_cmd_inputs_1b():
-    """ additional input with negative position """
+    """additional input with negative position"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -82,8 +94,36 @@ def test_shell_cmd_inputs_1b():
     assert shelly.cmdline == "executable inp-1 arg"
 
 
+def test_shell_cmd_inputs_1_st():
+    """additional input with provided position, checking cmdline when splitter"""
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=str,
+                    metadata={"position": 1, "help_string": "inp1", "argstr": ""},
+                ),
+            )
+        ],
+        bases=(ShellSpec,),
+    )
+
+    shelly = ShellCommandTask(
+        name="shelly",
+        executable="executable",
+        args="arg",
+        inpA=["inp1", "inp2"],
+        input_spec=my_input_spec,
+    ).split("inpA")
+    # cmdline should be a list
+    # assert shelly.cmdline[0] == "executable inp1 arg"
+    # assert shelly.cmdline[1] == "executable inp2 arg"
+
+
 def test_shell_cmd_inputs_2():
-    """ additional inputs with provided positions """
+    """additional inputs with provided positions"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -113,7 +153,7 @@ def test_shell_cmd_inputs_2():
 
 
 def test_shell_cmd_inputs_2a():
-    """ additional inputs without provided positions """
+    """additional inputs without provided positions"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -135,7 +175,7 @@ def test_shell_cmd_inputs_2a():
 
 
 def test_shell_cmd_inputs_2_err():
-    """ additional inputs with provided positions (exception due to the duplication)"""
+    """additional inputs with provided positions (exception due to the duplication)"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -158,15 +198,46 @@ def test_shell_cmd_inputs_2_err():
     )
 
     shelly = ShellCommandTask(
-        executable="executable", inpA="inp1", inpB="inp1", input_spec=my_input_spec
+        executable="executable", inpA="inp1", inpB="inp2", input_spec=my_input_spec
     )
     with pytest.raises(Exception) as e:
         shelly.cmdline
     assert "1 is already used" in str(e.value)
 
 
+def test_shell_cmd_inputs_2_noerr():
+    """additional inputs with provided positions
+    (duplication of the position doesn't lead to error, since only one field has value)
+    """
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=str,
+                    metadata={"position": 1, "help_string": "inpA", "argstr": ""},
+                ),
+            ),
+            (
+                "inpB",
+                attr.ib(
+                    type=str,
+                    metadata={"position": 1, "help_string": "inpB", "argstr": ""},
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    shelly = ShellCommandTask(
+        executable="executable", inpA="inp1", input_spec=my_input_spec
+    )
+    shelly.cmdline
+
+
 def test_shell_cmd_inputs_3():
-    """ additional inputs: positive pos, negative pos and  no pos """
+    """additional inputs: positive pos, negative pos and  no pos"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -197,12 +268,12 @@ def test_shell_cmd_inputs_3():
         inpC="inpNone",
         input_spec=my_input_spec,
     )
-    # input without position shoild be between positive an negative positions
+    # input without position should be between positive an negative positions
     assert shelly.cmdline == "executable inp1 inpNone inp-1"
 
 
 def test_shell_cmd_inputs_argstr_1():
-    """ additional string inputs with argstr """
+    """additional string inputs with argstr"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -225,7 +296,7 @@ def test_shell_cmd_inputs_argstr_1():
 
 
 def test_shell_cmd_inputs_argstr_2():
-    """ additional bool inputs with argstr """
+    """additional bool inputs with argstr"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -249,7 +320,7 @@ def test_shell_cmd_inputs_argstr_2():
 
 
 def test_shell_cmd_inputs_list_1():
-    """ providing list as an additional input, no sep, no argstr """
+    """providing list as an additional input, no sep, no argstr"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -272,7 +343,7 @@ def test_shell_cmd_inputs_list_1():
 
 
 def test_shell_cmd_inputs_list_2():
-    """ providing list as an additional input, no sep, but argstr """
+    """providing list as an additional input, no sep, but argstr"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -294,7 +365,7 @@ def test_shell_cmd_inputs_list_2():
 
 
 def test_shell_cmd_inputs_list_3():
-    """ providing list as an additional input, no sep, argstr with ..."""
+    """providing list as an additional input, no sep, argstr with ..."""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -317,7 +388,7 @@ def test_shell_cmd_inputs_list_3():
 
 
 def test_shell_cmd_inputs_list_sep_1():
-    """ providing list as an additional input:, sep, no argstr"""
+    """providing list as an additional input:, sep, no argstr"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -345,7 +416,7 @@ def test_shell_cmd_inputs_list_sep_1():
 
 
 def test_shell_cmd_inputs_list_sep_2():
-    """ providing list as an additional input:, sep, and argstr"""
+    """providing list as an additional input:, sep, and argstr"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -372,8 +443,36 @@ def test_shell_cmd_inputs_list_sep_2():
     assert shelly.cmdline == "executable -v aaa,bbb,ccc"
 
 
-def test_shell_cmd_inputs_sep_3():
-    """ providing list as an additional input:, sep, argstr with ..."""
+def test_shell_cmd_inputs_list_sep_2a():
+    """providing list as an additional input:, sep, and argstr with f-string"""
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "sep": ",",
+                        "argstr": "-v {inpA}",
+                    },
+                ),
+            )
+        ],
+        bases=(ShellSpec,),
+    )
+
+    shelly = ShellCommandTask(
+        executable="executable", inpA=["aaa", "bbb", "ccc"], input_spec=my_input_spec
+    )
+    # a flag is used once
+    assert shelly.cmdline == "executable -v aaa,bbb,ccc"
+
+
+def test_shell_cmd_inputs_list_sep_3():
+    """providing list as an additional input:, sep, argstr with ..."""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -400,8 +499,36 @@ def test_shell_cmd_inputs_sep_3():
     assert shelly.cmdline == "executable -v aaa, -v bbb, -v ccc"
 
 
+def test_shell_cmd_inputs_list_sep_3a():
+    """providing list as an additional input:, sep, argstr with ... and f-string"""
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "sep": ",",
+                        "argstr": "-v {inpA}...",
+                    },
+                ),
+            )
+        ],
+        bases=(ShellSpec,),
+    )
+
+    shelly = ShellCommandTask(
+        executable="executable", inpA=["aaa", "bbb", "ccc"], input_spec=my_input_spec
+    )
+    # a flag is repeated
+    assert shelly.cmdline == "executable -v aaa, -v bbb, -v ccc"
+
+
 def test_shell_cmd_inputs_sep_4():
-    """ providing 1-el list as an additional input:, sep, argstr with ..., """
+    """providing 1-el list as an additional input:, sep, argstr with ...,"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -428,7 +555,7 @@ def test_shell_cmd_inputs_sep_4():
 
 
 def test_shell_cmd_inputs_sep_4a():
-    """ providing str instead of list as an additional input:, sep, argstr with ..."""
+    """providing str instead of list as an additional input:, sep, argstr with ..."""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -455,7 +582,7 @@ def test_shell_cmd_inputs_sep_4a():
 
 
 def test_shell_cmd_inputs_format_1():
-    """ additional inputs with argstr that has string formatting"""
+    """additional inputs with argstr that has string formatting"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -481,7 +608,7 @@ def test_shell_cmd_inputs_format_1():
 
 
 def test_shell_cmd_inputs_format_2():
-    """ additional inputs with argstr that has string formatting and ..."""
+    """additional inputs with argstr that has string formatting and ..."""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -506,8 +633,34 @@ def test_shell_cmd_inputs_format_2():
     assert shelly.cmdline == "executable -v el_1 -v el_2"
 
 
+def test_shell_cmd_inputs_format_3():
+    """adding float formatting for argstr with input field"""
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=float,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "-v {inpA:.5f}",
+                    },
+                ),
+            )
+        ],
+        bases=(ShellSpec,),
+    )
+
+    shelly = ShellCommandTask(
+        executable="executable", inpA=0.007, input_spec=my_input_spec
+    )
+    assert shelly.cmdline == "executable -v 0.00700"
+
+
 def test_shell_cmd_inputs_mandatory_1():
-    """ additional inputs with mandatory=True"""
+    """additional inputs with mandatory=True"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -533,8 +686,54 @@ def test_shell_cmd_inputs_mandatory_1():
     assert "mandatory" in str(e.value)
 
 
+def test_shell_cmd_inputs_not_given_1():
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "arg1",
+                attr.ib(
+                    type=MultiInputObj,
+                    metadata={
+                        "argstr": "--arg1",
+                        "help_string": "Command line argument 1",
+                    },
+                ),
+            ),
+            (
+                "arg2",
+                attr.ib(
+                    type=MultiInputObj,
+                    metadata={
+                        "argstr": "--arg2",
+                        "help_string": "Command line argument 2",
+                    },
+                ),
+            ),
+            (
+                "arg3",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "argstr": "--arg3",
+                        "help_string": "Command line argument 3",
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+    shelly = ShellCommandTask(
+        name="shelly", executable="executable", input_spec=my_input_spec
+    )
+
+    shelly.inputs.arg2 = "argument2"
+
+    assert shelly.cmdline == f"executable --arg2 argument2"
+
+
 def test_shell_cmd_inputs_template_1():
-    """ additional inputs, one uses output_file_template (and argstr)"""
+    """additional inputs, one uses output_file_template (and argstr)"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -577,7 +776,7 @@ def test_shell_cmd_inputs_template_1():
 
 
 def test_shell_cmd_inputs_template_1a():
-    """ additional inputs, one uses output_file_template (without argstr)"""
+    """additional inputs, one uses output_file_template (without argstr)"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -616,7 +815,7 @@ def test_shell_cmd_inputs_template_1a():
 
 # TODO: after deciding how we use requires/templates
 def test_shell_cmd_inputs_template_2():
-    """ additional inputs, one uses output_file_template (and argstr, but input not provided)"""
+    """additional inputs, one uses output_file_template (and argstr, but input not provided)"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -651,7 +850,7 @@ def test_shell_cmd_inputs_template_2():
 
 
 def test_shell_cmd_inputs_template_3():
-    """ additional inputs with output_file_template and an additional
+    """additional inputs with output_file_template and an additional
     read-only fields that combine two outputs together in the command line
     """
     my_input_spec = SpecInfo(
@@ -730,7 +929,7 @@ def test_shell_cmd_inputs_template_3():
 
 
 def test_shell_cmd_inputs_template_3a():
-    """ additional inputs with output_file_template and an additional
+    """additional inputs with output_file_template and an additional
     read-only fields that combine two outputs together in the command line
     testing a different order within the input spec
     """
@@ -811,7 +1010,7 @@ def test_shell_cmd_inputs_template_3a():
 
 # TODO: after deciding how we use requires/templates
 def test_shell_cmd_inputs_template_4():
-    """ additional inputs with output_file_template and an additional
+    """additional inputs with output_file_template and an additional
     read-only fields that combine two outputs together in the command line
     one output_file_template can't be resolved - no inpB is provided
     """
@@ -882,7 +1081,7 @@ def test_shell_cmd_inputs_template_4():
 
 
 def test_shell_cmd_inputs_template_5_ex():
-    """ checking if the exception is raised for read-only fields when input is set"""
+    """checking if the exception is raised for read-only fields when input is set"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -911,10 +1110,10 @@ def test_shell_cmd_inputs_template_5_ex():
 
 
 def test_shell_cmd_inputs_template_6():
-    """ additional inputs with output_file_template that has type ty.Union[str, bool]
-        no default is set, so if nothing is provided as an input, the output  is used
-        whenever the template can be formatted
-        (the same way as for templates that has type=str)
+    """additional inputs with output_file_template that has type ty.Union[str, bool]
+    no default is set, so if nothing is provided as an input, the output  is used
+    whenever the template can be formatted
+    (the same way as for templates that has type=str)
     """
     my_input_spec = SpecInfo(
         name="Input",
@@ -974,9 +1173,9 @@ def test_shell_cmd_inputs_template_6():
 
 
 def test_shell_cmd_inputs_template_6a():
-    """ additional inputs with output_file_template that has type ty.Union[str, bool]
-        and default is set to False,
-        so if nothing is provided as an input, the output is not used
+    """additional inputs with output_file_template that has type ty.Union[str, bool]
+    and default is set to False,
+    so if nothing is provided as an input, the output is not used
     """
     my_input_spec = SpecInfo(
         name="Input",
@@ -1036,7 +1235,7 @@ def test_shell_cmd_inputs_template_6a():
 
 
 def test_shell_cmd_inputs_template_7(tmpdir):
-    """ additional inputs uses output_file_template with a suffix (no extension)
+    """additional inputs uses output_file_template with a suffix (no extension)
     no keep_extension is used
     """
     my_input_spec = SpecInfo(
@@ -1084,8 +1283,8 @@ def test_shell_cmd_inputs_template_7(tmpdir):
 
 
 def test_shell_cmd_inputs_template_7a(tmpdir):
-    """ additional inputs uses output_file_template with a suffix (no extension)
-        keep_extension is True (as default)
+    """additional inputs uses output_file_template with a suffix (no extension)
+    keep_extension is True (as default)
     """
     my_input_spec = SpecInfo(
         name="Input",
@@ -1133,7 +1332,7 @@ def test_shell_cmd_inputs_template_7a(tmpdir):
 
 
 def test_shell_cmd_inputs_template_7b(tmpdir):
-    """ additional inputs uses output_file_template with a suffix (no extension)
+    """additional inputs uses output_file_template with a suffix (no extension)
     keep extension is False (so the extension is removed when creating the output)
     """
     my_input_spec = SpecInfo(
@@ -1227,9 +1426,464 @@ def test_shell_cmd_inputs_template_8(tmpdir):
     )
 
 
+def test_shell_cmd_inputs_template_9(tmpdir):
+    """additional inputs, one uses output_file_template with two fields:
+    one File and one ints - the output should be recreated from the template
+    """
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "inpInt",
+                attr.ib(
+                    type=int,
+                    metadata={
+                        "position": 2,
+                        "help_string": "inp int",
+                        "argstr": "-i",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 3,
+                        "help_string": "outA",
+                        "argstr": "-o",
+                        "output_file_template": "{inpA}_{inpInt}_out.txt",
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    inpA_file = tmpdir.join("inpA.t")
+    inpA_file.write("content")
+
+    shelly = ShellCommandTask(
+        executable="executable", input_spec=my_input_spec, inpA=inpA_file, inpInt=3
+    )
+
+    assert (
+        shelly.cmdline
+        == f"executable {tmpdir.join('inpA.t')} -i 3 -o {str(shelly.output_dir / 'inpA_3_out.txt')}"
+    )
+    # checking if outA in the output fields
+    assert shelly.output_names == ["return_code", "stdout", "stderr", "outA"]
+
+
+def test_shell_cmd_inputs_template_9a(tmpdir):
+    """additional inputs, one uses output_file_template with two fields:
+    one file and one string without extension - should be fine
+    """
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "inpStr",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 2,
+                        "help_string": "inp str",
+                        "argstr": "-i",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 3,
+                        "help_string": "outA",
+                        "argstr": "-o",
+                        "output_file_template": "{inpA}_{inpStr}_out.txt",
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    inpA_file = tmpdir.join("inpA.t")
+    inpA_file.write("content")
+
+    shelly = ShellCommandTask(
+        executable="executable", input_spec=my_input_spec, inpA=inpA_file, inpStr="hola"
+    )
+
+    assert (
+        shelly.cmdline
+        == f"executable {tmpdir.join('inpA.t')} -i hola -o {str(shelly.output_dir / 'inpA_hola_out.txt')}"
+    )
+    # checking if outA in the output fields
+    assert shelly.output_names == ["return_code", "stdout", "stderr", "outA"]
+
+
+def test_shell_cmd_inputs_template_9b_err(tmpdir):
+    """output_file_template with two fields that are both Files,
+    an exception should be raised
+    """
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "inpFile",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "position": 2,
+                        "help_string": "inp file",
+                        "argstr": "-i",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 3,
+                        "help_string": "outA",
+                        "argstr": "-o",
+                        "output_file_template": "{inpA}_{inpFile}_out.txt",
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    inpA_file = tmpdir.join("inpA.t")
+    inpA_file.write("content")
+
+    inpFile_file = tmpdir.join("inpFile.t")
+    inpFile_file.write("content")
+
+    shelly = ShellCommandTask(
+        executable="executable",
+        input_spec=my_input_spec,
+        inpA=inpA_file,
+        inpFile=inpFile_file,
+    )
+    # the template has two files so the exception should be raised
+    with pytest.raises(Exception, match="can't have multiple paths"):
+        shelly.cmdline
+
+
+def test_shell_cmd_inputs_template_9c_err(tmpdir):
+    """output_file_template with two fields: a file and a string with extension,
+    that should be used as an additional file and the exception should be raised
+    """
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "inpStr",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 2,
+                        "help_string": "inp str with extension",
+                        "argstr": "-i",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 3,
+                        "help_string": "outA",
+                        "argstr": "-o",
+                        "output_file_template": "{inpA}_{inpStr}_out.txt",
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    inpA_file = tmpdir.join("inpA.t")
+    inpA_file.write("content")
+
+    shelly = ShellCommandTask(
+        executable="executable",
+        input_spec=my_input_spec,
+        inpA=inpA_file,
+        inpStr="hola.txt",
+    )
+    # inptStr has an extension so should be treated as a second file in the template formatting
+    # and the exception should be raised
+    with pytest.raises(Exception, match="can't have multiple paths"):
+        shelly.cmdline
+
+
+def test_shell_cmd_inputs_template_10():
+    """output_file_template uses a float field with formatting"""
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=float,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "{inpA:.1f}",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 2,
+                        "help_string": "outA",
+                        "argstr": "-o",
+                        "output_file_template": "file_{inpA:.1f}_out",
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    shelly = ShellCommandTask(
+        executable="executable", input_spec=my_input_spec, inpA=3.3456
+    )
+    # outA has argstr in the metadata fields, so it's a part of the command line
+    # the full path will be use din the command line
+    assert (
+        shelly.cmdline == f"executable 3.3 -o {str(shelly.output_dir / 'file_3.3_out')}"
+    )
+    # checking if outA in the output fields
+    assert shelly.output_names == ["return_code", "stdout", "stderr", "outA"]
+
+
+def test_shell_cmd_inputs_template_function_1():
+    """one input field uses output_file_template that is a simple function
+    this can be easily done by simple template as in test_shell_cmd_inputs_template_1
+    """
+
+    # a function that return an output template
+    def template_fun(inputs):
+        return "{inpA}_out"
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 2,
+                        "help_string": "outA",
+                        "argstr": "-o",
+                        "output_file_template": template_fun,
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    shelly = ShellCommandTask(
+        executable="executable", input_spec=my_input_spec, inpA="inpA"
+    )
+
+    assert shelly.cmdline == f"executable inpA -o {str(shelly.output_dir / 'inpA_out')}"
+
+
+def test_shell_cmd_inputs_template_function_2():
+    """one input field uses output_file_template that is a function,
+    depending on a value of an input it returns different template
+    """
+
+    # a function that return an output template that depends on value of the input
+    def template_fun(inputs):
+        if inputs.inpB % 2 == 0:
+            return "{inpA}_even"
+        else:
+            return "{inpA}_odd"
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "inpB",
+                attr.ib(
+                    type=int,
+                    metadata={
+                        "help_string": "inpB",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 2,
+                        "help_string": "outA",
+                        "argstr": "-o",
+                        "output_file_template": template_fun,
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    shelly = ShellCommandTask(
+        executable="executable",
+        input_spec=my_input_spec,
+        inpA="inpA",
+        inpB=1,
+    )
+
+    assert shelly.cmdline == f"executable inpA -o {str(shelly.output_dir / 'inpA_odd')}"
+
+
+def test_shell_cmd_inputs_template_1_st():
+    """additional inputs, one uses output_file_template (and argstr)
+    testing cmdline when splitter defined
+    """
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 2,
+                        "help_string": "outA",
+                        "argstr": "-o",
+                        "output_file_template": "{inpA}_out",
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    inpA = ["inpA_1", "inpA_2"]
+    shelly = ShellCommandTask(
+        name="f",
+        executable="executable",
+        input_spec=my_input_spec,
+        inpA=inpA,
+    ).split("inpA")
+
+    # cmdline_list = shelly.cmdline
+    # assert len(cmdline_list) == 2
+    # for i in range(2):
+    #     path_out = Path(shelly.output_dir[i]) / f"{inpA[i]}_out"
+    #     assert cmdline_list[i] == f"executable {inpA[i]} -o {str(path_out)}"
+
+
 # TODO: after deciding how we use requires/templates
 def test_shell_cmd_inputs_di(tmpdir, use_validator):
-    """ example from #279 """
+    """example from #279"""
     my_input_spec = SpecInfo(
         name="Input",
         fields=[
@@ -1426,7 +2080,7 @@ def test_shell_cmd_inputs_di(tmpdir, use_validator):
         == f"DenoiseImage -i {tmpdir.join('a_file.ext')} -s 1 -p 1 -r 2 -o [{str(shelly.output_dir / 'a_file_out.ext')}]"
     )
 
-    # input file name, noiseImage is set to True, so template is used in the utput
+    # input file name, noiseImage is set to True, so template is used in the output
     shelly = ShellCommandTask(
         executable="DenoiseImage",
         inputImageFilename=my_input_file,
@@ -1479,3 +2133,105 @@ def test_shell_cmd_inputs_di(tmpdir, use_validator):
             image_dimensionality=5,
         )
     assert "value of image_dimensionality" in str(excinfo.value)
+
+
+# tests with XOR in input metadata
+
+
+class SimpleTaskXor(ShellCommandTask):
+    input_fields = [
+        (
+            "input_1",
+            str,
+            {
+                "help_string": "help",
+                "mandatory": True,
+                "xor": ("input_1", "input_2", "input_3"),
+            },
+        ),
+        (
+            "input_2",
+            bool,
+            {
+                "help_string": "help",
+                "mandatory": True,
+                "argstr": "--i2",
+                "xor": ("input_1", "input_2", "input_3"),
+            },
+        ),
+        (
+            "input_3",
+            bool,
+            {
+                "help_string": "help",
+                "mandatory": True,
+                "xor": ("input_1", "input_2", "input_3"),
+            },
+        ),
+    ]
+    task_input_spec = SpecInfo(name="Input", fields=input_fields, bases=(ShellSpec,))
+    task_output_fields = []
+    task_output_spec = SpecInfo(
+        name="Output", fields=task_output_fields, bases=(ShellOutSpec,)
+    )
+
+    input_spec = task_input_spec
+    output_spec = task_output_spec
+    executable = "cmd"
+
+
+def test_task_inputs_mandatory_with_xOR_one_mandatory_is_OK():
+    """input spec with mandatory inputs"""
+    task = SimpleTaskXor()
+    task.inputs.input_1 = "Input1"
+    task.inputs.input_2 = attr.NOTHING
+    task.inputs.check_fields_input_spec()
+
+
+def test_task_inputs_mandatory_with_xOR_one_mandatory_out_3_is_OK():
+    """input spec with mandatory inputs"""
+    task = SimpleTaskXor()
+    task.inputs.input_1 = attr.NOTHING
+    task.inputs.input_2 = attr.NOTHING
+    task.inputs.input_3 = True
+    task.inputs.check_fields_input_spec()
+
+
+def test_task_inputs_mandatory_with_xOR_zero_mandatory_raises_error():
+    """input spec with mandatory inputs"""
+    task = SimpleTaskXor()
+    task.inputs.input_1 = attr.NOTHING
+    task.inputs.input_2 = attr.NOTHING
+    with pytest.raises(Exception) as excinfo:
+        task.inputs.check_fields_input_spec()
+    assert "input_1 is mandatory, but no value provided" in str(excinfo.value)
+    assert excinfo.type is AttributeError
+
+
+def test_task_inputs_mandatory_with_xOR_two_mandatories_raises_error():
+    """input spec with mandatory inputs"""
+    task = SimpleTaskXor()
+    task.inputs.input_1 = "Input1"
+    task.inputs.input_2 = True
+
+    with pytest.raises(Exception) as excinfo:
+        task.inputs.check_fields_input_spec()
+    assert "input_2 is mutually exclusive with ('input_1', 'input_2'" in str(
+        excinfo.value
+    )
+    assert excinfo.type is AttributeError
+
+
+def test_task_inputs_mandatory_with_xOR_3_mandatories_raises_error():
+    """input spec with mandatory inputs"""
+    task = SimpleTaskXor()
+    task.inputs.input_1 = "Input1"
+    task.inputs.input_2 = True
+    task.inputs.input_3 = False
+
+    with pytest.raises(Exception) as excinfo:
+        task.inputs.check_fields_input_spec()
+    assert "input_2 is mutually exclusive with ('input_1', 'input_2', 'input_3'" in str(
+        excinfo.value
+    )
+    assert excinfo.type is AttributeError
