@@ -17,38 +17,42 @@ need_singularity = pytest.mark.skipif(
     shutil.which("singularity") is None, reason="no singularity available"
 )
 
+need_slurm = pytest.mark.skipif(
+    not bool(shutil.which("sbatch")), reason="no singularity available"
+)
+
 
 @need_singularity
 def test_singularity_1_nosubm(tmpdir):
-    """ simple command in a container, a default bindings and working directory is added
-        no submitter
+    """simple command in a container, a default bindings and working directory is added
+    no submitter
     """
     cmd = "pwd"
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
     singu = SingularityTask(name="singu", executable=cmd, image=image, cache_dir=tmpdir)
-    assert singu.inputs.image == "library://sylabsed/linux/alpine"
+    assert singu.inputs.image == "docker://alpine"
     assert singu.inputs.container == "singularity"
     assert (
         singu.cmdline
-        == f"singularity exec -B {singu.output_dir}:/output_pydra:rw {image} {cmd}"
+        == f"singularity exec -B {singu.output_dir}:/output_pydra:rw --pwd /output_pydra {image} {cmd}"
     )
 
     res = singu()
-    assert "SingularityTask" in res.output.stdout
+    assert "output_pydra" in res.output.stdout
     assert res.output.return_code == 0
 
 
 @need_singularity
 def test_singularity_2_nosubm(tmpdir):
-    """ a command with arguments, cmd and args given as executable
-        no submitter
+    """a command with arguments, cmd and args given as executable
+    no submitter
     """
     cmd = ["echo", "hail", "pydra"]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
     singu = SingularityTask(name="singu", executable=cmd, image=image, cache_dir=tmpdir)
     assert (
         singu.cmdline
-        == f"singularity exec -B {singu.output_dir}:/output_pydra:rw {image} {' '.join(cmd)}"
+        == f"singularity exec -B {singu.output_dir}:/output_pydra:rw --pwd /output_pydra {image} {' '.join(cmd)}"
     )
 
     res = singu()
@@ -58,15 +62,15 @@ def test_singularity_2_nosubm(tmpdir):
 
 @need_singularity
 def test_singularity_2(plugin, tmpdir):
-    """ a command with arguments, cmd and args given as executable
-        using submitter
+    """a command with arguments, cmd and args given as executable
+    using submitter
     """
     cmd = ["echo", "hail", "pydra"]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
     singu = SingularityTask(name="singu", executable=cmd, image=image, cache_dir=tmpdir)
     assert (
         singu.cmdline
-        == f"singularity exec -B {singu.output_dir}:/output_pydra:rw {image} {' '.join(cmd)}"
+        == f"singularity exec -B {singu.output_dir}:/output_pydra:rw --pwd /output_pydra {image} {' '.join(cmd)}"
     )
 
     with Submitter(plugin=plugin) as sub:
@@ -78,11 +82,11 @@ def test_singularity_2(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_2_singuflag(plugin, tmpdir):
-    """ a command with arguments, cmd and args given as executable
-        using ShellComandTask with container_info=("singularity", image)
+    """a command with arguments, cmd and args given as executable
+    using ShellComandTask with container_info=("singularity", image)
     """
     cmd = ["echo", "hail", "pydra"]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
     shingu = ShellCommandTask(
         name="shingu",
         executable=cmd,
@@ -91,7 +95,7 @@ def test_singularity_2_singuflag(plugin, tmpdir):
     )
     assert (
         shingu.cmdline
-        == f"singularity exec -B {shingu.output_dir}:/output_pydra:rw {image} {' '.join(cmd)}"
+        == f"singularity exec -B {shingu.output_dir}:/output_pydra:rw --pwd /output_pydra {image} {' '.join(cmd)}"
     )
 
     with Submitter(plugin=plugin) as sub:
@@ -103,19 +107,19 @@ def test_singularity_2_singuflag(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_2a(plugin, tmpdir):
-    """ a command with arguments, using executable and args
-        using submitter
+    """a command with arguments, using executable and args
+    using submitter
     """
     cmd_exec = "echo"
     cmd_args = ["hail", "pydra"]
     # separate command into exec + args
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
     singu = SingularityTask(
         name="singu", executable=cmd_exec, args=cmd_args, image=image, cache_dir=tmpdir
     )
     assert (
         singu.cmdline
-        == f"singularity exec -B {singu.output_dir}:/output_pydra:rw {image} {cmd_exec} {' '.join(cmd_args)}"
+        == f"singularity exec -B {singu.output_dir}:/output_pydra:rw --pwd /output_pydra {image} {cmd_exec} {' '.join(cmd_args)}"
     )
 
     with Submitter(plugin=plugin) as sub:
@@ -126,14 +130,15 @@ def test_singularity_2a(plugin, tmpdir):
 
 
 @need_singularity
+@pytest.mark.skip(reason="we probably don't want to support bindings as an input")
 def test_singularity_3(plugin, tmpdir):
-    """ a simple command in container with bindings,
-        creating directory in tmp dir and checking if it is in the container
+    """a simple command in container with bindings,
+    creating directory in tmp dir and checking if it is in the container
     """
     # creating a new directory
     tmpdir.mkdir("new_dir")
     cmd = ["ls", "/tmp_dir"]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
     singu = SingularityTask(name="singu", executable=cmd, image=image, cache_dir=tmpdir)
     # binding tmp directory to the container
     singu.inputs.bindings = [(str(tmpdir), "/tmp_dir", "ro")]
@@ -147,15 +152,16 @@ def test_singularity_3(plugin, tmpdir):
 
 
 @need_singularity
+@pytest.mark.skip(reason="we probably don't want to support bindings as an input")
 def test_singularity_3_singuflag(plugin, tmpdir):
-    """ a simple command in container with bindings,
-        creating directory in tmp dir and checking if it is in the container
-        using ShellComandTask with container_info=("singularity", image)
+    """a simple command in container with bindings,
+    creating directory in tmp dir and checking if it is in the container
+    using ShellComandTask with container_info=("singularity", image)
     """
     # creating a new directory
     tmpdir.mkdir("new_dir")
     cmd = ["ls", "/tmp_dir"]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
     shingu = SingularityTask(
         name="singu",
         executable=cmd,
@@ -174,15 +180,16 @@ def test_singularity_3_singuflag(plugin, tmpdir):
 
 
 @need_singularity
+@pytest.mark.skip(reason="we probably don't want to support bindings as an input")
 def test_singularity_3_singuflagbind(plugin, tmpdir):
-    """ a simple command in container with bindings,
-        creating directory in tmp dir and checking if it is in the container
-        using ShellComandTask with container_info=("singularity", image, bindings)
+    """a simple command in container with bindings,
+    creating directory in tmp dir and checking if it is in the container
+    using ShellComandTask with container_info=("singularity", image, bindings)
     """
     # creating a new directory
     tmpdir.mkdir("new_dir")
     cmd = ["ls", "/tmp_dir"]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
     shingu = SingularityTask(
         name="singu",
         executable=cmd,
@@ -203,29 +210,29 @@ def test_singularity_3_singuflagbind(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_st_1(plugin, tmpdir):
-    """ commands without arguments in container
-        splitter = executable
+    """commands without arguments in container
+    splitter = executable
     """
     cmd = ["pwd", "ls"]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
     singu = SingularityTask(
         name="singu", executable=cmd, image=image, cache_dir=tmpdir
     ).split("executable")
     assert singu.state.splitter == "singu.executable"
 
     res = singu(plugin=plugin)
-    assert "SingularityTask" in res[0].output.stdout
+    assert "/output_pydra" in res[0].output.stdout
     assert res[1].output.stdout == ""
     assert res[0].output.return_code == res[1].output.return_code == 0
 
 
 @need_singularity
 def test_singularity_st_2(plugin, tmpdir):
-    """ command with arguments in docker, checking the distribution
-        splitter = image
+    """command with arguments in docker, checking the distribution
+    splitter = image
     """
     cmd = ["cat", "/etc/issue"]
-    image = ["library://sylabsed/linux/alpine", "library://sylabsed/examples/lolcow"]
+    image = ["docker://alpine", "docker://ubuntu"]
     singu = SingularityTask(
         name="singu", executable=cmd, image=image, cache_dir=tmpdir
     ).split("image")
@@ -239,32 +246,52 @@ def test_singularity_st_2(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_st_3(plugin, tmpdir):
-    """ outer splitter image and executable
-    """
+    """outer splitter image and executable"""
     cmd = ["pwd", ["cat", "/etc/issue"]]
-    image = ["library://sylabsed/linux/alpine", "library://sylabsed/examples/lolcow"]
+    image = ["docker://alpine", "docker://ubuntu"]
     singu = SingularityTask(
         name="singu", executable=cmd, image=image, cache_dir=tmpdir
     ).split(["image", "executable"])
     assert singu.state.splitter == ["singu.image", "singu.executable"]
     res = singu(plugin=plugin)
 
-    assert "SingularityTask" in res[0].output.stdout
+    assert "/output_pydra" in res[0].output.stdout
     assert "Alpine" in res[1].output.stdout
-    assert "SingularityTask" in res[2].output.stdout
+    assert "/output_pydra" in res[2].output.stdout
     assert "Ubuntu" in res[3].output.stdout
 
 
 @need_singularity
+@need_slurm
+@pytest.mark.xfail(
+    reason="slurm can complain if the number of submitted jobs exceeds the limit"
+)
+@pytest.mark.parametrize("n", [10, 50, 100])
+def test_singularity_st_4(tmpdir, n):
+    """splitter over args (checking bigger splitters if slurm available)"""
+    args_n = list(range(n))
+    image = "docker://alpine"
+    singu = SingularityTask(
+        name="singu", executable="echo", image=image, cache_dir=tmpdir, args=args_n
+    ).split("args")
+    assert singu.state.splitter == "singu.args"
+    res = singu(plugin="slurm")
+    assert "1" in res[1].output.stdout
+    assert str(n - 1) in res[-1].output.stdout
+    assert res[0].output.return_code == res[1].output.return_code == 0
+
+
+@need_singularity
+@pytest.mark.skip(reason="we probably don't want to support bindings as an input")
 def test_wf_singularity_1(plugin, tmpdir):
-    """ a workflow with two connected task
-        the first one read the file that is bounded to the container,
-        the second uses echo
+    """a workflow with two connected task
+    the first one read the file that is bounded to the container,
+    the second uses echo
     """
     with open(tmpdir.join("file_pydra.txt"), "w") as f:
         f.write("hello from pydra")
 
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
     wf = Workflow(name="wf", input_spec=["cmd1", "cmd2"], cache_dir=tmpdir)
     wf.inputs.cmd1 = ["cat", "/tmp_dir/file_pydra.txt"]
     wf.inputs.cmd2 = ["echo", "message from the previous task:"]
@@ -297,15 +324,16 @@ def test_wf_singularity_1(plugin, tmpdir):
 
 @need_docker
 @need_singularity
+@pytest.mark.skip(reason="we probably don't want to support bindings as an input")
 def test_wf_singularity_1a(plugin, tmpdir):
-    """ a workflow with two connected task - using both containers: Docker and Singul.
-        the first one read the file that is bounded to the container,
-        the second uses echo
+    """a workflow with two connected task - using both containers: Docker and Singul.
+    the first one read the file that is bounded to the container,
+    the second uses echo
     """
     with open(tmpdir.join("file_pydra.txt"), "w") as f:
         f.write("hello from pydra")
 
-    image_sing = "library://sylabsed/linux/alpine"
+    image_sing = "docker://alpine"
     image_doc = "ubuntu"
     wf = Workflow(name="wf", input_spec=["cmd1", "cmd2"], cache_dir=tmpdir)
     wf.inputs.cmd1 = ["cat", "/tmp_dir/file_pydra.txt"]
@@ -343,11 +371,11 @@ def test_wf_singularity_1a(plugin, tmpdir):
 @need_singularity
 def test_singularity_outputspec_1(plugin, tmpdir):
     """
-        customised output_spec, adding files to the output, providing specific pathname
-        output_path is automatically added to the bindings
+    customised output_spec, adding files to the output, providing specific pathname
+    output_path is automatically added to the bindings
     """
     cmd = ["touch", "newfile_tmp.txt"]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     my_output_spec = SpecInfo(
         name="Output",
@@ -375,13 +403,13 @@ def test_singularity_outputspec_1(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_inputspec_1(plugin, tmpdir):
-    """ a simple customized input spec for singularity task """
+    """a simple customized input spec for singularity task"""
     filename = str(tmpdir.join("file_pydra.txt"))
     with open(filename, "w") as f:
         f.write("hello from pydra")
 
     cmd = "cat"
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     my_input_spec = SpecInfo(
         name="Input",
@@ -418,15 +446,15 @@ def test_singularity_inputspec_1(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_inputspec_1a(plugin, tmpdir):
-    """ a simple customized input spec for singularity task
-        a default value is used
+    """a simple customized input spec for singularity task
+    a default value is used
     """
     filename = str(tmpdir.join("file_pydra.txt"))
     with open(filename, "w") as f:
         f.write("hello from pydra")
 
     cmd = "cat"
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     my_input_spec = SpecInfo(
         name="Input",
@@ -458,7 +486,7 @@ def test_singularity_inputspec_1a(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_inputspec_2(plugin, tmpdir):
-    """ a customized input spec with two fields for singularity task """
+    """a customized input spec with two fields for singularity task"""
     filename_1 = tmpdir.join("file_pydra.txt")
     with open(filename_1, "w") as f:
         f.write("hello from pydra\n")
@@ -468,7 +496,7 @@ def test_singularity_inputspec_2(plugin, tmpdir):
         f.write("have a nice one")
 
     cmd = "cat"
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     my_input_spec = SpecInfo(
         name="Input",
@@ -516,8 +544,8 @@ def test_singularity_inputspec_2(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_inputspec_2a_except(plugin, tmpdir):
-    """ a customized input spec with two fields
-        first one uses a default, and second doesn't - raises a dataclass exception
+    """a customized input spec with two fields
+    first one uses a default, and second doesn't - raises a dataclass exception
     """
     filename_1 = tmpdir.join("file_pydra.txt")
     with open(filename_1, "w") as f:
@@ -527,7 +555,7 @@ def test_singularity_inputspec_2a_except(plugin, tmpdir):
         f.write("have a nice one")
 
     cmd = "cat"
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     # the field with default value can't be before value without default
     my_input_spec = SpecInfo(
@@ -575,9 +603,9 @@ def test_singularity_inputspec_2a_except(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_inputspec_2a(plugin, tmpdir):
-    """ a customized input spec with two fields
-        first one uses a default value,
-        this is fine even if the second field is not using any defaults
+    """a customized input spec with two fields
+    first one uses a default value,
+    this is fine even if the second field is not using any defaults
     """
     filename_1 = tmpdir.join("file_pydra.txt")
     with open(filename_1, "w") as f:
@@ -587,7 +615,7 @@ def test_singularity_inputspec_2a(plugin, tmpdir):
         f.write("have a nice one")
 
     cmd = "cat"
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     # if you want set default in the first field you can use default_value in metadata
     my_input_spec = SpecInfo(
@@ -636,16 +664,16 @@ def test_singularity_inputspec_2a(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_cmd_inputspec_copyfile_1(plugin, tmpdir):
-    """ shelltask changes a file in place,
-        adding copyfile=True to the file-input from input_spec
-        hardlink or copy in the output_dir should be created
+    """shelltask changes a file in place,
+    adding copyfile=True to the file-input from input_spec
+    hardlink or copy in the output_dir should be created
     """
     file = tmpdir.join("file_pydra.txt")
     with open(file, "w") as f:
         f.write("hello from pydra\n")
 
     cmd = ["sed", "-is", "s/hello/hi/"]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     my_input_spec = SpecInfo(
         name="Input",
@@ -700,8 +728,8 @@ def test_singularity_cmd_inputspec_copyfile_1(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_inputspec_state_1(plugin, tmpdir):
-    """ a customised input spec for a singularity file with a splitter,
-        splitter is on files
+    """a customised input spec for a singularity file with a splitter,
+    splitter is on files
     """
     filename_1 = tmpdir.join("file_pydra.txt")
     with open(filename_1, "w") as f:
@@ -712,7 +740,7 @@ def test_singularity_inputspec_state_1(plugin, tmpdir):
 
     cmd = "cat"
     filename = [str(filename_1), str(filename_2)]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     my_input_spec = SpecInfo(
         name="Input",
@@ -750,9 +778,9 @@ def test_singularity_inputspec_state_1(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_inputspec_state_1b(plugin, tmpdir):
-    """ a customised input spec for a singularity file with a splitter,
-        files from the input spec have the same path in the local os and the container,
-        so hash is calculated and the test works fine
+    """a customised input spec for a singularity file with a splitter,
+    files from the input spec have the same path in the local os and the container,
+    so hash is calculated and the test works fine
     """
     file_1 = tmpdir.join("file_pydra.txt")
     file_2 = tmpdir.join("file_nice.txt")
@@ -763,7 +791,7 @@ def test_singularity_inputspec_state_1b(plugin, tmpdir):
 
     cmd = "cat"
     filename = [str(file_1), str(file_2)]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     my_input_spec = SpecInfo(
         name="Input",
@@ -801,13 +829,13 @@ def test_singularity_inputspec_state_1b(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_wf_inputspec_1(plugin, tmpdir):
-    """ a customized input spec for workflow with singularity tasks """
+    """a customized input spec for workflow with singularity tasks"""
     filename = tmpdir.join("file_pydra.txt")
     with open(filename, "w") as f:
         f.write("hello from pydra")
 
     cmd = "cat"
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     my_input_spec = SpecInfo(
         name="Input",
@@ -853,7 +881,7 @@ def test_singularity_wf_inputspec_1(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_wf_state_inputspec_1(plugin, tmpdir):
-    """ a customized input spec for workflow with singularity tasks that has a state"""
+    """a customized input spec for workflow with singularity tasks that has a state"""
     file_1 = tmpdir.join("file_pydra.txt")
     file_2 = tmpdir.join("file_nice.txt")
     with open(file_1, "w") as f:
@@ -863,7 +891,7 @@ def test_singularity_wf_state_inputspec_1(plugin, tmpdir):
 
     cmd = "cat"
     filename = [str(file_1), str(file_2)]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     my_input_spec = SpecInfo(
         name="Input",
@@ -911,7 +939,7 @@ def test_singularity_wf_state_inputspec_1(plugin, tmpdir):
 
 @need_singularity
 def test_singularity_wf_ndst_inputspec_1(plugin, tmpdir):
-    """ a customized input spec for workflow with singularity tasks with states"""
+    """a customized input spec for workflow with singularity tasks with states"""
     file_1 = tmpdir.join("file_pydra.txt")
     file_2 = tmpdir.join("file_nice.txt")
     with open(file_1, "w") as f:
@@ -921,7 +949,7 @@ def test_singularity_wf_ndst_inputspec_1(plugin, tmpdir):
 
     cmd = "cat"
     filename = [str(file_1), str(file_2)]
-    image = "library://sylabsed/linux/alpine"
+    image = "docker://alpine"
 
     my_input_spec = SpecInfo(
         name="Input",
