@@ -2,12 +2,26 @@ import typing as ty
 import os, sys
 import attr
 import pytest
-
+import cloudpickle as cp
+from pathlib import Path
+import re
+import json
+import glob as glob
 from ... import mark
+from ..core import Workflow
 from ..task import AuditFlag, ShellCommandTask, DockerTask, SingularityTask
 from ...utils.messenger import FileMessenger, PrintMessenger, collect_messages
-from .utils import gen_basic_wf, use_validator
-from ..specs import MultiInputObj, MultiOutputObj, SpecInfo, FunctionSpec, BaseSpec
+from .utils import gen_basic_wf, use_validator, Submitter
+from ..specs import (
+    MultiInputObj,
+    MultiOutputObj,
+    SpecInfo,
+    FunctionSpec,
+    BaseSpec,
+    ShellSpec,
+    File,
+)
+from ..helpers import hash_file
 
 no_win = pytest.mark.skipif(
     sys.platform.startswith("win"),
@@ -27,7 +41,7 @@ def test_output():
 
 
 def test_name_conflict():
-    """ raise error if task name conflicts with a class attribute or method"""
+    """raise error if task name conflicts with a class attribute or method"""
     with pytest.raises(ValueError) as excinfo1:
         nn = funaddtwo(name="split", a=3)
     assert "Cannot use names of attributes or methods" in str(excinfo1.value)
@@ -37,7 +51,7 @@ def test_name_conflict():
 
 
 def test_numpy(use_validator):
-    """ checking if mark.task works for numpy functions"""
+    """checking if mark.task works for numpy functions"""
     np = pytest.importorskip("numpy")
     fft = mark.annotate({"a": np.ndarray, "return": np.ndarray})(np.fft.fft)
     fft = mark.task(fft)()
@@ -101,7 +115,7 @@ def test_annotated_func(use_validator):
 
 
 def test_annotated_func_multreturn(use_validator):
-    """ the function has two elements in the return statement"""
+    """the function has two elements in the return statement"""
 
     @mark.task
     def testfunc(
@@ -140,7 +154,7 @@ def test_annotated_func_multreturn(use_validator):
 
 
 def test_annotated_input_func_1(use_validator):
-    """ the function with annotated input (float)"""
+    """the function with annotated input (float)"""
 
     @mark.task
     def testfunc(a: float):
@@ -151,7 +165,7 @@ def test_annotated_input_func_1(use_validator):
 
 
 def test_annotated_input_func_2(use_validator):
-    """ the function with annotated input (int, but float provided)"""
+    """the function with annotated input (int, but float provided)"""
 
     @mark.task
     def testfunc(a: int):
@@ -162,7 +176,7 @@ def test_annotated_input_func_2(use_validator):
 
 
 def test_annotated_input_func_2a(use_validator):
-    """ the function with annotated input (int, but float provided)"""
+    """the function with annotated input (int, but float provided)"""
 
     @mark.task
     def testfunc(a: int):
@@ -174,7 +188,7 @@ def test_annotated_input_func_2a(use_validator):
 
 
 def test_annotated_input_func_3(use_validator):
-    """ the function with annotated input (list)"""
+    """the function with annotated input (list)"""
 
     @mark.task
     def testfunc(a: list):
@@ -185,7 +199,7 @@ def test_annotated_input_func_3(use_validator):
 
 
 def test_annotated_input_func_3a():
-    """ the function with annotated input (list of floats)"""
+    """the function with annotated input (list of floats)"""
 
     @mark.task
     def testfunc(a: ty.List[float]):
@@ -196,7 +210,7 @@ def test_annotated_input_func_3a():
 
 
 def test_annotated_input_func_3b(use_validator):
-    """ the function with annotated input
+    """the function with annotated input
     (list of floats - int and float provided, should be fine)
     """
 
@@ -209,7 +223,7 @@ def test_annotated_input_func_3b(use_validator):
 
 
 def test_annotated_input_func_3c_excep(use_validator):
-    """ the function with annotated input
+    """the function with annotated input
     (list of ints - int and float provided, should raise an error)
     """
 
@@ -222,7 +236,7 @@ def test_annotated_input_func_3c_excep(use_validator):
 
 
 def test_annotated_input_func_4(use_validator):
-    """ the function with annotated input (dictionary)"""
+    """the function with annotated input (dictionary)"""
 
     @mark.task
     def testfunc(a: dict):
@@ -233,7 +247,7 @@ def test_annotated_input_func_4(use_validator):
 
 
 def test_annotated_input_func_4a(use_validator):
-    """ the function with annotated input (dictionary of floats)"""
+    """the function with annotated input (dictionary of floats)"""
 
     @mark.task
     def testfunc(a: ty.Dict[str, float]):
@@ -244,7 +258,7 @@ def test_annotated_input_func_4a(use_validator):
 
 
 def test_annotated_input_func_4b_excep(use_validator):
-    """ the function with annotated input (dictionary of ints, but float provided)"""
+    """the function with annotated input (dictionary of ints, but float provided)"""
 
     @mark.task
     def testfunc(a: ty.Dict[str, int]):
@@ -255,9 +269,9 @@ def test_annotated_input_func_4b_excep(use_validator):
 
 
 def test_annotated_input_func_5(use_validator):
-    """ the function with annotated more complex input type (ty.List in ty.Dict)
-        the validator should simply check if values of dict are lists
-        so no error for 3.5
+    """the function with annotated more complex input type (ty.List in ty.Dict)
+    the validator should simply check if values of dict are lists
+    so no error for 3.5
     """
 
     @mark.task
@@ -269,8 +283,8 @@ def test_annotated_input_func_5(use_validator):
 
 
 def test_annotated_input_func_5a_except(use_validator):
-    """ the function with annotated more complex input type (ty.Dict in ty.Dict)
-        list is provided as a dict value (instead a dict), so error is raised
+    """the function with annotated more complex input type (ty.Dict in ty.Dict)
+    list is provided as a dict value (instead a dict), so error is raised
     """
 
     @mark.task
@@ -282,8 +296,8 @@ def test_annotated_input_func_5a_except(use_validator):
 
 
 def test_annotated_input_func_6(use_validator):
-    """ the function with annotated more complex input type (ty.Union in ty.Dict)
-        the validator should unpack values from the Union
+    """the function with annotated more complex input type (ty.Union in ty.Dict)
+    the validator should unpack values from the Union
     """
 
     @mark.task
@@ -295,8 +309,8 @@ def test_annotated_input_func_6(use_validator):
 
 
 def test_annotated_input_func_6a_excep(use_validator):
-    """ the function with annotated more complex input type (ty.Union in ty.Dict)
-        the validator should unpack values from the Union and raise an error for 3.5
+    """the function with annotated more complex input type (ty.Union in ty.Dict)
+    the validator should unpack values from the Union and raise an error for 3.5
     """
 
     @mark.task
@@ -308,9 +322,9 @@ def test_annotated_input_func_6a_excep(use_validator):
 
 
 def test_annotated_input_func_7(use_validator):
-    """ the function with annotated input (float)
-        the task has a splitter, so list of float is provided
-        it should work, the validator tries to guess if this is a field with a splitter
+    """the function with annotated input (float)
+    the task has a splitter, so list of float is provided
+    it should work, the validator tries to guess if this is a field with a splitter
     """
 
     @mark.task
@@ -322,8 +336,8 @@ def test_annotated_input_func_7(use_validator):
 
 
 def test_annotated_input_func_7a_excep(use_validator):
-    """ the function with annotated input (int) and splitter
-        list of float provided - should raise an error (list of int would be fine)
+    """the function with annotated input (int) and splitter
+    list of float provided - should raise an error (list of int would be fine)
     """
 
     @mark.task
@@ -335,8 +349,8 @@ def test_annotated_input_func_7a_excep(use_validator):
 
 
 def test_annotated_input_func_8():
-    """ the function with annotated input as MultiInputObj
-        a single value is provided and should be converted to a list
+    """the function with annotated input as MultiInputObj
+    a single value is provided and should be converted to a list
     """
 
     @mark.task
@@ -350,8 +364,8 @@ def test_annotated_input_func_8():
 
 
 def test_annotated_input_func_8a():
-    """ the function with annotated input as MultiInputObj
-        a 1-el list is provided so shouldn't be changed
+    """the function with annotated input as MultiInputObj
+    a 1-el list is provided so shouldn't be changed
     """
 
     @mark.task
@@ -365,9 +379,9 @@ def test_annotated_input_func_8a():
 
 
 def test_annotated_input_func_8b():
-    """ the function with annotated input as MultiInputObj
-        a single value is provided after initial. the task
-        (input should still be converted to a list)
+    """the function with annotated input as MultiInputObj
+    a single value is provided after initial. the task
+    (input should still be converted to a list)
     """
 
     @mark.task
@@ -384,14 +398,14 @@ def test_annotated_input_func_8b():
 
 def test_annotated_func_multreturn_exception(use_validator):
     """function has two elements in the return statement,
-        but three element provided in the spec - should raise an error
+    but three element provided in the spec - should raise an error
     """
 
     @mark.task
     def testfunc(
         a: float,
     ) -> ty.NamedTuple(
-        "Output", [("fractional", float), ("integer", int), ("whoknows", int)]
+        "Output", [("fractional", float), ("integer", int), ("who_knows", int)]
     ):
         import math
 
@@ -528,8 +542,8 @@ def test_halfannotated_func_multrun_returnlist():
 
 
 def test_notannotated_func_multreturn():
-    """ no annotation and multiple values are returned
-        all elements should be returned as a tuple ans set to "out"
+    """no annotation and multiple values are returned
+    all elements should be returned as a tuple and set to "out"
     """
 
     @mark.task
@@ -548,7 +562,7 @@ def test_notannotated_func_multreturn():
 
 
 def test_input_spec_func_1(use_validator):
-    """ the function w/o annotated, but input_spec is used """
+    """the function w/o annotated, but input_spec is used"""
 
     @mark.task
     def testfunc(a):
@@ -565,7 +579,7 @@ def test_input_spec_func_1(use_validator):
 
 
 def test_input_spec_func_1a_except(use_validator):
-    """ the function w/o annotated, but input_spec is used
+    """the function w/o annotated, but input_spec is used
     a TypeError is raised (float is provided instead of int)
     """
 
@@ -583,9 +597,9 @@ def test_input_spec_func_1a_except(use_validator):
 
 
 def test_input_spec_func_1b_except(use_validator):
-    """ the function w/o annotated, but input_spec is used
-     metadata checks raise an error
-     """
+    """the function w/o annotated, but input_spec is used
+    metadata checks raise an error
+    """
 
     @mark.task
     def testfunc(a):
@@ -606,7 +620,7 @@ def test_input_spec_func_1b_except(use_validator):
 
 
 def test_input_spec_func_1d_except(use_validator):
-    """ the function w/o annotated, but input_spec is used
+    """the function w/o annotated, but input_spec is used
     input_spec doesn't contain 'a' input, an error is raised
     """
 
@@ -621,7 +635,7 @@ def test_input_spec_func_1d_except(use_validator):
 
 
 def test_input_spec_func_2(use_validator):
-    """ the function with annotation, and the task has input_spec,
+    """the function with annotation, and the task has input_spec,
     input_spec changes the type of the input (so error is not raised)
     """
 
@@ -640,7 +654,7 @@ def test_input_spec_func_2(use_validator):
 
 
 def test_input_spec_func_2a(use_validator):
-    """ the function with annotation, and the task has input_spec,
+    """the function with annotation, and the task has input_spec,
     input_spec changes the type of the input (so error is not raised)
     using the shorter syntax
     """
@@ -660,7 +674,7 @@ def test_input_spec_func_2a(use_validator):
 
 
 def test_input_spec_func_3(use_validator):
-    """ the function w/o annotated, but input_spec is used
+    """the function w/o annotated, but input_spec is used
     additional keys (allowed_values) are used in metadata
     """
 
@@ -687,7 +701,7 @@ def test_input_spec_func_3(use_validator):
 
 
 def test_input_spec_func_3a_except(use_validator):
-    """ the function w/o annotated, but input_spec is used
+    """the function w/o annotated, but input_spec is used
     allowed_values is used in metadata and the ValueError is raised
     """
 
@@ -714,7 +728,7 @@ def test_input_spec_func_3a_except(use_validator):
 
 
 def test_input_spec_func_4(use_validator):
-    """ the function with a default value for b
+    """the function with a default value for b
     but b is set as mandatory in the input_spec, so error is raised if not provided
     """
 
@@ -747,7 +761,7 @@ def test_input_spec_func_4(use_validator):
 
 
 def test_input_spec_func_4a(use_validator):
-    """ the function with a default value for b and metadata in the input_spec
+    """the function with a default value for b and metadata in the input_spec
     has a different default value, so value from the function is overwritten
     """
 
@@ -775,8 +789,8 @@ def test_input_spec_func_4a(use_validator):
 
 
 def test_input_spec_func_5():
-    """ the FunctionTask with input_spec, a input has MultiInputObj type
-        a single value is provided and should be converted to a list
+    """the FunctionTask with input_spec, a input has MultiInputObj type
+    a single value is provided and should be converted to a list
     """
 
     @mark.task
@@ -798,7 +812,7 @@ def test_input_spec_func_5():
 
 
 def test_output_spec_func_1(use_validator):
-    """ the function w/o annotated, but output_spec is used """
+    """the function w/o annotated, but output_spec is used"""
 
     @mark.task
     def testfunc(a):
@@ -816,8 +830,8 @@ def test_output_spec_func_1(use_validator):
 
 
 def test_output_spec_func_1a_except(use_validator):
-    """ the function w/o annotated, but output_spec is used
-        float returned instead of int - TypeError
+    """the function w/o annotated, but output_spec is used
+    float returned instead of int - TypeError
     """
 
     @mark.task
@@ -836,7 +850,7 @@ def test_output_spec_func_1a_except(use_validator):
 
 
 def test_output_spec_func_2(use_validator):
-    """ the function w/o annotated, but output_spec is used
+    """the function w/o annotated, but output_spec is used
     output_spec changes the type of the output (so error is not raised)
     """
 
@@ -856,7 +870,7 @@ def test_output_spec_func_2(use_validator):
 
 
 def test_output_spec_func_2a(use_validator):
-    """ the function w/o annotated, but output_spec is used
+    """the function w/o annotated, but output_spec is used
     output_spec changes the type of the output (so error is not raised)
     using a shorter syntax
     """
@@ -877,7 +891,7 @@ def test_output_spec_func_2a(use_validator):
 
 
 def test_output_spec_func_3(use_validator):
-    """ the function w/o annotated, but output_spec is used
+    """the function w/o annotated, but output_spec is used
     MultiOutputObj is used, output is a 2-el list, so converter doesn't do anything
     """
 
@@ -902,7 +916,7 @@ def test_output_spec_func_3(use_validator):
 
 
 def test_output_spec_func_4(use_validator):
-    """ the function w/o annotated, but output_spec is used
+    """the function w/o annotated, but output_spec is used
     MultiOutputObj is used, output is a 1el list, so converter return the element
     """
 
@@ -936,7 +950,7 @@ def test_exception_func():
 
 
 def test_result_none_1():
-    """ checking if None is properly returned as the result"""
+    """checking if None is properly returned as the result"""
 
     @mark.task
     def fun_none(x):
@@ -948,7 +962,7 @@ def test_result_none_1():
 
 
 def test_result_none_2():
-    """ checking if None is properly set for all outputs """
+    """checking if None is properly set for all outputs"""
 
     @mark.task
     def fun_none(x) -> (ty.Any, ty.Any):
@@ -972,15 +986,275 @@ def test_audit_prov(tmpdir, use_validator):
 
     # saving the audit message into the file
     funky = testfunc(a=2, audit_flags=AuditFlag.PROV, messengers=FileMessenger())
-    message_path = tmpdir / funky.checksum / "messages"
     funky.cache_dir = tmpdir
-    funky.messenger_args = dict(message_dir=message_path)
     funky()
+    # this should be the default loctaion
+    message_path = tmpdir / funky.checksum / "messages"
+    assert (tmpdir / funky.checksum / "messages").exists()
 
     collect_messages(tmpdir / funky.checksum, message_path, ld_op="compact")
     assert (tmpdir / funky.checksum / "messages.jsonld").exists()
 
 
+def test_audit_task(tmpdir):
+    @mark.task
+    def testfunc(a: int, b: float = 0.1) -> ty.NamedTuple("Output", [("out", float)]):
+        return a + b
+
+    from glob import glob
+
+    funky = testfunc(a=2, audit_flags=AuditFlag.PROV, messengers=FileMessenger())
+    funky.cache_dir = tmpdir
+    funky()
+    message_path = tmpdir / funky.checksum / "messages"
+
+    for file in glob(str(message_path) + "/*.jsonld"):
+        with open(file, "r") as f:
+            data = json.load(f)
+            if "@type" in data:
+                if "AssociatedWith" in data:
+                    assert "testfunc" in data["Label"]
+
+            if "@type" in data:
+                if data["@type"] == "input":
+                    assert None == data["Label"]
+            if "AssociatedWith" in data:
+                assert None == data["AssociatedWith"]
+
+    # assert any(json_content)
+
+
+def test_audit_shellcommandtask(tmpdir):
+    args = "-l"
+    shelly = ShellCommandTask(
+        name="shelly",
+        executable="ls",
+        args=args,
+        audit_flags=AuditFlag.PROV,
+        messengers=FileMessenger(),
+    )
+
+    from glob import glob
+
+    shelly.cache_dir = tmpdir
+    shelly()
+    message_path = tmpdir / shelly.checksum / "messages"
+    # go through each jsonld file in message_path and check if the label field exists
+
+    command_content = []
+
+    for file in glob(str(message_path) + "/*.jsonld"):
+        with open(file, "r") as f:
+            data = json.load(f)
+
+            if "@type" in data:
+                if "AssociatedWith" in data:
+                    assert "shelly" in data["Label"]
+
+            if "@type" in data:
+                if data["@type"] == "input":
+                    assert data["Label"] == None
+
+            if "Command" in data:
+                command_content.append(True)
+                assert "ls -l" == data["Command"]
+
+    assert any(command_content)
+
+
+def test_audit_shellcommandtask_file(tmpdir):
+    # sourcery skip: use-fstring-for-concatenation
+    import glob
+    import shutil
+
+    # create test.txt file with "This is a test" in it in the tmpdir
+    # create txt file in cwd
+    with open("test.txt", "w") as f:
+        f.write("This is a test")
+
+    with open("test2.txt", "w") as f:
+        f.write("This is a test")
+
+    # copy the test.txt file to the tmpdir
+    shutil.copy("test.txt", tmpdir)
+    shutil.copy("test2.txt", tmpdir)
+
+    cmd = "cat"
+    file_in = tmpdir / "test.txt"
+    file_in_2 = tmpdir / "test2.txt"
+    test_file_hash = hash_file(file_in)
+    test_file_hash_2 = hash_file(file_in_2)
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "in_file",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "position": 1,
+                        "argstr": "",
+                        "help_string": "text",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "in_file_2",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "position": 2,
+                        "argstr": "",
+                        "help_string": "text",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+    shelly = ShellCommandTask(
+        name="shelly",
+        in_file=file_in,
+        in_file_2=file_in_2,
+        input_spec=my_input_spec,
+        executable=cmd,
+        audit_flags=AuditFlag.PROV,
+        messengers=FileMessenger(),
+    )
+    shelly.cache_dir = tmpdir
+    shelly()
+    message_path = tmpdir / shelly.checksum / "messages"
+    for file in glob.glob(str(message_path) + "/*.jsonld"):
+        with open(file, "r") as x:
+            data = json.load(x)
+            if "@type" in data:
+                if data["@type"] == "input":
+                    if data["Label"] == "in_file":
+                        assert data["AtLocation"] == str(file_in)
+                        assert data["digest"] == test_file_hash
+                    if data["Label"] == "in_file_2":
+                        assert data["AtLocation"] == str(file_in_2)
+                        assert data["digest"] == test_file_hash_2
+
+
+def test_audit_shellcommandtask_version(tmpdir):
+    import subprocess as sp
+
+    version_cmd = sp.run("less --version", shell=True, stdout=sp.PIPE).stdout.decode(
+        "utf-8"
+    )
+    version_cmd = version_cmd.splitlines()[0]
+    cmd = "less"
+    shelly = ShellCommandTask(
+        name="shelly",
+        executable=cmd,
+        args="test_task.py",
+        audit_flags=AuditFlag.PROV,
+        messengers=FileMessenger(),
+    )
+
+    import glob
+
+    shelly.cache_dir = tmpdir
+    shelly()
+    message_path = tmpdir / shelly.checksum / "messages"
+    # go through each jsonld file in message_path and check if the label field exists
+    version_content = []
+    for file in glob.glob(str(message_path) + "/*.jsonld"):
+        with open(file, "r") as f:
+            data = json.load(f)
+            if "AssociatedWith" in data:
+                if version_cmd in data["AssociatedWith"]:
+                    version_content.append(True)
+
+    assert any(version_content)
+
+
+def test_audit_prov_messdir_1(tmpdir, use_validator):
+    """customized messenger dir"""
+
+    @mark.task
+    def testfunc(a: int, b: float = 0.1) -> ty.NamedTuple("Output", [("out", float)]):
+        return a + b
+
+    # printing the audit message
+    funky = testfunc(a=1, audit_flags=AuditFlag.PROV, messengers=PrintMessenger())
+    funky.cache_dir = tmpdir
+    funky()
+
+    # saving the audit message into the file
+    funky = testfunc(a=2, audit_flags=AuditFlag.PROV, messengers=FileMessenger())
+    # user defined path
+    message_path = tmpdir / funky.checksum / "my_messages"
+    funky.cache_dir = tmpdir
+    # providing messenger_dir for audit
+    funky.audit.messenger_args = dict(message_dir=message_path)
+    funky()
+    assert (tmpdir / funky.checksum / "my_messages").exists()
+
+    collect_messages(tmpdir / funky.checksum, message_path, ld_op="compact")
+    assert (tmpdir / funky.checksum / "messages.jsonld").exists()
+
+
+def test_audit_prov_messdir_2(tmpdir, use_validator):
+    """customized messenger dir in init"""
+
+    @mark.task
+    def testfunc(a: int, b: float = 0.1) -> ty.NamedTuple("Output", [("out", float)]):
+        return a + b
+
+    # printing the audit message
+    funky = testfunc(a=1, audit_flags=AuditFlag.PROV, messengers=PrintMessenger())
+    funky.cache_dir = tmpdir
+    funky()
+
+    # user defined path (doesn't depend on checksum, can be defined before init)
+    message_path = tmpdir / "my_messages"
+    # saving the audit message into the file
+    funky = testfunc(
+        a=2,
+        audit_flags=AuditFlag.PROV,
+        messengers=FileMessenger(),
+        messenger_args=dict(message_dir=message_path),
+    )
+    funky.cache_dir = tmpdir
+    # providing messenger_dir for audit
+    funky()
+    assert (tmpdir / "my_messages").exists()
+
+    collect_messages(tmpdir, message_path, ld_op="compact")
+    assert (tmpdir / "messages.jsonld").exists()
+
+
+def test_audit_prov_wf(tmpdir, use_validator):
+    """FileMessenger for wf"""
+
+    @mark.task
+    def testfunc(a: int, b: float = 0.1) -> ty.NamedTuple("Output", [("out", float)]):
+        return a + b
+
+    wf = Workflow(
+        name="wf",
+        input_spec=["x"],
+        cache_dir=tmpdir,
+        audit_flags=AuditFlag.PROV,
+        messengers=FileMessenger(),
+    )
+    wf.add(testfunc(name="testfunc", a=wf.lzin.x))
+    wf.set_output([("out", wf.testfunc.lzout.out)])
+    wf.inputs.x = 2
+
+    wf(plugin="cf")
+    # default path
+    message_path = tmpdir / wf.checksum / "messages"
+    assert message_path.exists()
+
+    collect_messages(tmpdir / wf.checksum, message_path, ld_op="compact")
+    assert (tmpdir / wf.checksum / "messages.jsonld").exists()
+
+
 def test_audit_all(tmpdir, use_validator):
     @mark.task
     def testfunc(a: int, b: float = 0.1) -> ty.NamedTuple("Output", [("out", float)]):
@@ -989,12 +1263,12 @@ def test_audit_all(tmpdir, use_validator):
     funky = testfunc(a=2, audit_flags=AuditFlag.ALL, messengers=FileMessenger())
     message_path = tmpdir / funky.checksum / "messages"
     funky.cache_dir = tmpdir
-    funky.messenger_args = dict(message_dir=message_path)
+    funky.audit.messenger_args = dict(message_dir=message_path)
     funky()
     from glob import glob
 
     assert len(glob(str(tmpdir / funky.checksum / "proc*.log"))) == 1
-    assert len(glob(str(message_path / "*.jsonld"))) == 6
+    assert len(glob(str(message_path / "*.jsonld"))) == 7
 
     # commented out to speed up testing
     collect_messages(tmpdir / funky.checksum, message_path, ld_op="compact")
@@ -1036,19 +1310,20 @@ def test_docker_cmd(tmpdir):
         docky.cmdline
         == f"docker run --rm -v {docky.output_dir}:/output_pydra:rw -w /output_pydra busybox pwd"
     )
-    docky.inputs.container_xargs = ["--rm -it"]
+    docky.inputs.container_xargs = ["--rm", "-it"]
     assert (
         docky.cmdline
         == f"docker run --rm -it -v {docky.output_dir}:/output_pydra:rw -w /output_pydra busybox pwd"
     )
-    docky.inputs.bindings = [
-        ("/local/path", "/container/path", "ro"),
-        ("/local2", "/container2", None),
-    ]
-    assert docky.cmdline == (
-        "docker run --rm -it -v /local/path:/container/path:ro"
-        f" -v /local2:/container2:rw -v {docky.output_dir}:/output_pydra:rw -w /output_pydra busybox pwd"
-    )
+    # TODO: we probably don't want to support container_path
+    # docky.inputs.bindings = [
+    #     ("/local/path", "/container/path", "ro"),
+    #     ("/local2", "/container2", None),
+    # ]
+    # assert docky.cmdline == (
+    #     "docker run --rm -it -v /local/path:/container/path:ro"
+    #     f" -v /local2:/container2:rw -v {docky.output_dir}:/output_pydra:rw -w /output_pydra busybox pwd"
+    # )
 
 
 @no_win
@@ -1058,16 +1333,17 @@ def test_singularity_cmd(tmpdir):
     singu = SingularityTask(name="singi", executable="pwd", image=image)
     assert (
         singu.cmdline
-        == f"singularity exec -B {singu.output_dir}:/output_pydra:rw {image} pwd"
-    )
-    singu.inputs.bindings = [
-        ("/local/path", "/container/path", "ro"),
-        ("/local2", "/container2", None),
-    ]
-    assert singu.cmdline == (
-        "singularity exec -B /local/path:/container/path:ro"
-        f" -B /local2:/container2:rw -B {singu.output_dir}:/output_pydra:rw {image} pwd"
+        == f"singularity exec -B {singu.output_dir}:/output_pydra:rw --pwd /output_pydra {image} pwd"
     )
+    # TODO: we probably don't want to support container_path
+    # singu.inputs.bindings = [
+    #     ("/local/path", "/container/path", "ro"),
+    #     ("/local2", "/container2", None),
+    # ]
+    # assert singu.cmdline == (
+    #     "singularity exec -B /local/path:/container/path:ro"
+    #     f" -B /local2:/container2:rw -B {singu.output_dir}:/output_pydra:rw --pwd /output_pydra {image} pwd"
+    # )
 
 
 def test_functask_callable(tmpdir):
@@ -1217,3 +1493,113 @@ def test_taskhooks_4(tmpdir, capsys):
     # only post run task hook should be called
     assert len(hook_messages) == 1
     assert "postrun task hook was called" in hook_messages[0]
+
+
+def test_traceback(tmpdir):
+    """checking if the error raised in a function is properly returned;
+    checking if there is an error filename in the error message that contains
+    full traceback including the line in the python function
+    """
+
+    @mark.task
+    def fun_error(x):
+        raise Exception("Error from the function")
+
+    task = fun_error(name="error", x=[3, 4], cache_dir=tmpdir).split("x")
+
+    with pytest.raises(Exception, match="from the function") as exinfo:
+        res = task()
+
+    # getting error file from the error message
+    error_file_match = str(exinfo.value).split("here: ")[-1].split("_error.pklz")[0]
+    error_file = Path(error_file_match) / "_error.pklz"
+    # checking if the file exists
+    assert error_file.exists()
+    # reading error message from the pickle file
+    error_tb = cp.loads(error_file.read_bytes())["error message"]
+    # the error traceback should be a list and should point to a specific line in the function
+    assert isinstance(error_tb, list)
+    assert "in fun_error" in error_tb[-2]
+
+
+def test_traceback_wf(tmpdir):
+    """checking if the error raised in a function is properly returned by a workflow;
+    checking if there is an error filename in the error message that contains
+    full traceback including the line in the python function
+    """
+
+    @mark.task
+    def fun_error(x):
+        raise Exception("Error from the function")
+
+    wf = Workflow(name="wf", input_spec=["x"], x=[3, 4], cache_dir=tmpdir).split("x")
+    wf.add(fun_error(name="error", x=wf.lzin.x))
+    wf.set_output([("out", wf.error.lzout.out)])
+
+    with pytest.raises(Exception, match="Task error raised an error") as exinfo:
+        res = wf()
+
+    # getting error file from the error message
+    error_file_match = str(exinfo.value).split("here: ")[-1].split("_error.pklz")[0]
+    error_file = Path(error_file_match) / "_error.pklz"
+    # checking if the file exists
+    assert error_file.exists()
+    # reading error message from the pickle file
+    error_tb = cp.loads(error_file.read_bytes())["error message"]
+    # the error traceback should be a list and should point to a specific line in the function
+    assert isinstance(error_tb, list)
+    assert "in fun_error" in error_tb[-2]
+
+
+def test_rerun_errored(tmpdir, capfd):
+    """Test rerunning a task containing errors.
+    Only the errored tasks should be rerun"""
+
+    @mark.task
+    def pass_odds(x):
+        if x % 2 == 0:
+            print(f"x%2 = {x % 2} (error)\n")
+            raise Exception("even error")
+        else:
+            print(f"x%2 = {x % 2}\n")
+            return x
+
+    task = pass_odds(name="pass_odds", x=[1, 2, 3, 4, 5], cache_dir=tmpdir).split("x")
+
+    with pytest.raises(Exception, match="even error"):
+        task()
+    with pytest.raises(Exception, match="even error"):
+        task()
+
+    out, err = capfd.readouterr()
+    stdout_lines = out.splitlines()
+
+    tasks_run = 0
+    errors_found = 0
+
+    for line in stdout_lines:
+        if "x%2" in line:
+            tasks_run += 1
+        if "(error)" in line:
+            errors_found += 1
+
+    # There should have been 5 messages of the form "x%2 = XXX" after calling task() the first time
+    # and another 2 messagers after calling the second time
+    assert tasks_run == 7
+    assert errors_found == 4
+
+
+@attr.s(auto_attribs=True)
+class A:
+    x: int
+
+
+def test_object_input():
+    """Test function tasks with object inputs"""
+
+    @mark.task
+    def testfunc(a: A):
+        return a.x
+
+    result = testfunc(a=A(x=7))()
+    assert result.output.out == 7
